from .argentina import Argentina as Argentina
from .barbados import Barbados as Barbados
from .brazil import (
    Brazil as Brazil,
    BrazilAcre as BrazilAcre,
    BrazilAlagoas as BrazilAlagoas,
    BrazilAmapa as BrazilAmapa,
    BrazilAmazonas as BrazilAmazonas,
    BrazilAracajuCity as BrazilAracajuCity,
    BrazilBahia as BrazilBahia,
    BrazilBankCalendar as BrazilBankCalendar,
    BrazilBelemCity as BrazilBelemCity,
    BrazilBeloHorizonteCity as BrazilBeloHorizonteCity,
    BrazilBoaVistaCity as BrazilBoaVistaCity,
    BrazilCampoGrandeCity as BrazilCampoGrandeCity,
    BrazilCariacicaCity as BrazilCariacicaCity,
    BrazilCeara as BrazilCeara,
    BrazilChapecoCity as BrazilChapecoCity,
    BrazilCuiabaCity as BrazilCuiabaCity,
    BrazilCuritibaCity as BrazilCuritibaCity,
    BrazilDistritoFederal as BrazilDistritoFederal,
    BrazilEspiritoSanto as BrazilEspiritoSanto,
    BrazilFlorianopolisCity as BrazilFlorianopolisCity,
    BrazilFortalezaCity as BrazilFortalezaCity,
    BrazilGoianiaCity as BrazilGoianiaCity,
    BrazilGoias as BrazilGoias,
    BrazilGuarapariCity as BrazilGuarapariCity,
    BrazilJoaoPessoaCity as BrazilJoaoPessoaCity,
    BrazilJoinvilleCity as BrazilJoinvilleCity,
    BrazilMacapaCity as BrazilMacapaCity,
    BrazilMaceioCity as BrazilMaceioCity,
    BrazilManausCity as BrazilManausCity,
    BrazilMaranhao as BrazilMaranhao,
    BrazilMatoGrosso as BrazilMatoGrosso,
    BrazilMatoGrossoDoSul as BrazilMatoGrossoDoSul,
    BrazilMinasGerais as BrazilMinasGerais,
    BrazilNatalCity as BrazilNatalCity,
    BrazilPalmasCity as BrazilPalmasCity,
    BrazilPara as BrazilPara,
    BrazilParaiba as BrazilParaiba,
    BrazilParana as BrazilParana,
    BrazilPernambuco as BrazilPernambuco,
    BrazilPiaui as BrazilPiaui,
    BrazilPortoAlegreCity as BrazilPortoAlegreCity,
    BrazilPortoVelhoCity as BrazilPortoVelhoCity,
    BrazilRecifeCity as BrazilRecifeCity,
    BrazilRioBrancoCity as BrazilRioBrancoCity,
    BrazilRioDeJaneiro as BrazilRioDeJaneiro,
    BrazilRioGrandeDoNorte as BrazilRioGrandeDoNorte,
    BrazilRioGrandeDoSul as BrazilRioGrandeDoSul,
    BrazilRondonia as BrazilRondonia,
    BrazilRoraima as BrazilRoraima,
    BrazilSalvadorCity as BrazilSalvadorCity,
    BrazilSantaCatarina as BrazilSantaCatarina,
    BrazilSaoPauloCity as BrazilSaoPauloCity,
    BrazilSaoPauloState as BrazilSaoPauloState,
    BrazilSergipe as BrazilSergipe,
    BrazilSerraCity as BrazilSerraCity,
    BrazilSorocabaCity as BrazilSorocabaCity,
    BrazilTeresinaCity as BrazilTeresinaCity,
    BrazilTocantins as BrazilTocantins,
    BrazilVilaVelhaCity as BrazilVilaVelhaCity,
    BrazilVitoriaCity as BrazilVitoriaCity,
)
from .canada import (
    Alberta as Alberta,
    BritishColumbia as BritishColumbia,
    Canada as Canada,
    Manitoba as Manitoba,
    NewBrunswick as NewBrunswick,
    Newfoundland as Newfoundland,
    NorthwestTerritories as NorthwestTerritories,
    NovaScotia as NovaScotia,
    Nunavut as Nunavut,
    Ontario as Ontario,
    PrinceEdwardIsland as PrinceEdwardIsland,
    Quebec as Quebec,
    Saskatchewan as Saskatchewan,
    Yukon as Yukon,
)
from .chile import Chile as Chile
from .colombia import Colombia as Colombia
from .el_salvador import ElSalvador as ElSalvador
from .mexico import Mexico as Mexico
from .panama import Panama as Panama
from .paraguay import Paraguay as Paraguay

__all__ = (
    # Brazil & its states.
    "Brazil",
    "BrazilAcre",
    "BrazilAlagoas",
    "BrazilAmapa",
    "BrazilAmazonas",
    "BrazilBahia",
    "BrazilCeara",
    "BrazilDistritoFederal",
    "BrazilEspiritoSanto",
    "BrazilGoias",
    "BrazilMaranhao",
    "BrazilMinasGerais",
    "BrazilMatoGrosso",
    "BrazilMatoGrossoDoSul",
    "BrazilPara",
    "BrazilParaiba",
    "BrazilPernambuco",
    "BrazilPiaui",
    "BrazilParana",
    "BrazilRioDeJaneiro",
    "BrazilRioGrandeDoNorte",
    "BrazilRioGrandeDoSul",
    "BrazilRondonia",
    "BrazilRoraima",
    "BrazilSantaCatarina",
    "BrazilSaoPauloState",
    "BrazilSaoPauloCity",
    "BrazilSergipe",
    "BrazilTocantins",
    "BrazilVitoriaCity",
    "BrazilVilaVelhaCity",
    "BrazilCariacicaCity",
    "BrazilGuarapariCity",
    "BrazilSerraCity",
    "BrazilBankCalendar",
    "BrazilRioBrancoCity",
    "BrazilMaceioCity",
    "BrazilManausCity",
    "BrazilMacapaCity",
    "BrazilSalvadorCity",
    "BrazilFortalezaCity",
    "BrazilGoianiaCity",
    "BrazilBeloHorizonteCity",
    "BrazilCampoGrandeCity",
    "BrazilCuiabaCity",
    "BrazilBelemCity",
    "BrazilJoaoPessoaCity",
    "BrazilRecifeCity",
    "BrazilTeresinaCity",
    "BrazilCuritibaCity",
    "BrazilNatalCity",
    "BrazilPortoVelhoCity",
    "BrazilBoaVistaCity",
    "BrazilPortoAlegreCity",
    "BrazilChapecoCity",
    "BrazilFlorianopolisCity",
    "BrazilJoinvilleCity",
    "BrazilAracajuCity",
    "BrazilSorocabaCity",
    "BrazilPalmasCity",
    # Canada
    "Canada",
    "Ontario",
    "Quebec",
    "BritishColumbia",
    "Alberta",
    "Saskatchewan",
    "Manitoba",
    "NewBrunswick",
    "NovaScotia",
    "PrinceEdwardIsland",
    "Newfoundland",
    "Yukon",
    "NorthwestTerritories",
    "Nunavut",
    # Other american countries
    "Barbados",
    "Chile",
    "Colombia",
    "Mexico",
    "Panama",
    "Paraguay",
    "Argentina",
    "ElSalvador",
)



from _typeshed import Unused
from collections.abc import Callable, Sequence, Sized
from typing import Any, Final, Literal, TypeVar

from Xlib.display import Display
from Xlib.protocol import display, rq
from Xlib.xobject import resource

_T = TypeVar("_T")
_S = TypeVar("_S", bound=Sized)

extname: Final = "RECORD"

FromServerTime: Final = 0x01
FromClientTime: Final = 0x02
FromClientSequence: Final = 0x04

CurrentClients: Final = 1
FutureClients: Final = 2
AllClients: Final = 3

FromServer: Final = 0
FromClient: Final = 1
ClientStarted: Final = 2
ClientDied: Final = 3
StartOfData: Final = 4
EndOfData: Final = 5
Record_Range8: rq.Struct
Record_Range16: rq.Struct
Record_ExtRange: rq.Struct
Record_Range: rq.Struct
Record_ClientInfo: rq.Struct

class RawField(rq.ValueField):
    structcode: None
    def pack_value(self, val: _S) -> tuple[_S, int, None]: ...  # type: ignore[override]
    def parse_binary_value(self, data: _T, display: Unused, length: Unused, format: Unused) -> tuple[_T, Literal[""]]: ...  # type: ignore[override]  # See: https://github.com/python-xlib/python-xlib/pull/249

class GetVersion(rq.ReplyRequest): ...

def get_version(self: Display | resource.Resource, major: int, minor: int) -> GetVersion: ...

class CreateContext(rq.Request): ...

def create_context(
    self: Display | resource.Resource,
    datum_flags: int,
    clients: Sequence[int],
    ranges: Sequence[
        tuple[
            tuple[int, int],
            tuple[int, int],
            tuple[int, int],
            tuple[int, int],
            tuple[int, int],
            tuple[int, int],
            tuple[int, int],
            bool,
            bool,
        ]
    ],
) -> int: ...

class RegisterClients(rq.Request): ...

def register_clients(
    self: Display | resource.Resource,
    context: int,
    element_header: int,
    clients: int,
    ranges: Sequence[
        tuple[
            tuple[int, int],
            tuple[int, int],
            tuple[int, int],
            tuple[int, int],
            tuple[int, int],
            tuple[int, int],
            tuple[int, int],
            bool,
            bool,
        ]
    ],
) -> None: ...

class UnregisterClients(rq.Request): ...

def unregister_clients(self: Display | resource.Resource, context: int, clients: Sequence[int]) -> None: ...

class GetContext(rq.ReplyRequest): ...

def get_context(self: Display | resource.Resource, context: int) -> GetContext: ...

class EnableContext(rq.ReplyRequest):
    def __init__(
        self,
        callback: Callable[[rq.DictWrapper | dict[str, Any]], Any],
        display: display.Display,
        defer: bool = False,
        *args: object | bool,
        **keys: object | bool,
    ) -> None: ...

def enable_context(
    self: Display | resource.Resource, context: int, callback: Callable[[rq.DictWrapper | dict[str, Any]], Any]
) -> None: ...

class DisableContext(rq.Request): ...

def disable_context(self: Display | resource.Resource, context: int) -> None: ...

class FreeContext(rq.Request): ...

def free_context(self: Display | resource.Resource, context: int) -> None: ...
def init(disp: Display, info: Unused) -> None: ...



from collections.abc import Callable
from datetime import datetime
from typing import Any, Literal
from typing_extensions import Self

from passlib.exc import (
    InvalidTokenError as InvalidTokenError,
    MalformedTokenError as MalformedTokenError,
    TokenError as TokenError,
    UsedTokenError as UsedTokenError,
)
from passlib.utils import SequenceMixin

class AppWallet:
    salt_size: int
    encrypt_cost: int
    default_tag: str | None
    def __init__(
        self,
        secrets: dict[int, str] | dict[int, bytes] | dict[str, str] | dict[str, bytes] | str | None = None,
        default_tag: str | None = None,
        encrypt_cost: int | None = None,
        secrets_path: str | None = None,
    ) -> None: ...
    @property
    def has_secrets(self) -> bool: ...
    def get_secret(self, tag: str) -> bytes: ...
    def encrypt_key(self, key: bytes) -> dict[str, Any]: ...
    def decrypt_key(self, enckey: dict[str, Any]) -> tuple[bytes, bool]: ...

class TOTP:
    min_json_version: int
    json_version: int
    wallet: AppWallet | None
    now: Callable[[], float]
    digits: int
    alg: str
    label: str | None
    issuer: str | None
    period: int
    changed: bool
    @classmethod
    def using(
        cls,
        digits: int | None = None,
        alg: Literal["sha1", "sha256", "sha512"] | None = None,
        period: int | None = None,
        issuer: str | None = None,
        wallet: AppWallet | None = None,
        now: Callable[[], float] | None = None,
        *,
        secrets: dict[int, str] | dict[int, bytes] | dict[str, str] | dict[str, bytes] | str | None = None,
        **kwds: Any,
    ) -> type[TOTP]: ...
    @classmethod
    def new(cls, **kwds: Any) -> Self: ...
    def __init__(
        self,
        key: str | bytes | None = None,
        format: str = "base32",
        new: bool = False,
        digits: int | None = None,
        alg: Literal["sha1", "sha256", "sha512"] | None = None,
        size: int | None = None,
        period: int | None = None,
        label: str | None = None,
        issuer: str | None = None,
        changed: bool = False,
        **kwds: Any,
    ) -> None: ...
    @property
    def key(self) -> bytes: ...
    @key.setter
    def key(self, value: bytes) -> None: ...
    @property
    def encrypted_key(self) -> dict[str, Any]: ...
    @encrypted_key.setter
    def encrypted_key(self, value: dict[str, Any]) -> None: ...
    @property
    def hex_key(self) -> str: ...
    @property
    def base32_key(self) -> str: ...
    def pretty_key(self, format: str = "base32", sep: str | Literal[False] = "-") -> str: ...
    @classmethod
    def normalize_time(cls, time: float | datetime | None) -> int: ...
    def normalize_token(self_or_cls, token: bytes | str | int) -> str: ...
    def generate(self, time: float | datetime | None = None) -> TotpToken: ...
    @classmethod
    def verify(
        cls,
        token: int | str,
        source: TOTP | dict[str, Any] | str | bytes,
        *,
        time: float | datetime | None = ...,
        window: int = ...,
        skew: int = ...,
        last_counter: int | None = ...,
    ) -> TotpMatch: ...
    def match(
        self,
        token: int | str,
        time: float | datetime | None = None,
        window: int = 30,
        skew: int = 0,
        last_counter: int | None = None,
    ) -> TotpMatch: ...
    @classmethod
    def from_source(cls, source: TOTP | dict[str, Any] | str | bytes) -> Self: ...
    @classmethod
    def from_uri(cls, uri: str) -> Self: ...
    def to_uri(self, label: str | None = None, issuer: str | None = None) -> str: ...
    @classmethod
    def from_json(cls, source: str | bytes) -> Self: ...
    def to_json(self, encrypt: bool | None = None) -> str: ...
    @classmethod
    def from_dict(cls, source: dict[str, Any]) -> Self: ...
    def to_dict(self, encrypt: bool | None = None) -> dict[str, Any]: ...

class TotpToken(SequenceMixin):
    totp: TOTP | None
    token: str | None
    counter: int | None
    def __init__(self, totp: TOTP, token: str, counter: int) -> None: ...
    @property
    def start_time(self) -> int: ...
    @property
    def expire_time(self) -> int: ...
    @property
    def remaining(self) -> float: ...
    @property
    def valid(self) -> bool: ...

class TotpMatch(SequenceMixin):
    totp: TOTP | None
    counter: int
    time: int
    window: int
    def __init__(self, totp: TOTP, counter: int, time: int, window: int = 30) -> None: ...
    @property
    def expected_counter(self) -> int: ...
    @property
    def skipped(self) -> int: ...
    @property
    def expire_time(self) -> int: ...
    @property
    def cache_seconds(self) -> int: ...
    @property
    def cache_time(self) -> int: ...

__all__ = [
    "AppWallet",
    "TOTP",
    "TokenError",
    "MalformedTokenError",
    "InvalidTokenError",
    "UsedTokenError",
    "TotpToken",
    "TotpMatch",
]



from _typeshed import Unused
from typing import TypedDict, type_check_only
from typing_extensions import TypeAlias

_SourceReferenceID: TypeAlias = int

@type_check_only
class Source(TypedDict, total=False):
    name: str
    path: str
    sourceReference: _SourceReferenceID

@type_check_only
class _LoadSourcesResult(TypedDict):
    sources: list[Source]

@type_check_only
class _SourceResult(TypedDict):
    content: str

def make_source(fullname: str, filename: str | None) -> Source: ...
def decode_source(source: Source) -> _SourceReferenceID: ...
def loaded_sources(**extra: Unused) -> _LoadSourcesResult: ...
def source(*, source: Source | None = None, sourceReference: _SourceReferenceID, **extra: Unused) -> _SourceResult: ...



from typing import Final

__version__: Final[str]

def makeEmptySquare(x, y, size, color): ...
def makeFilledSquare(x, y, size, color): ...
def makeFilledDiamond(x, y, size, color): ...
def makeEmptyCircle(x, y, size, color): ...
def makeFilledCircle(x, y, size, color): ...
def makeSmiley(x, y, size, color): ...



from typing import Final, Literal

EPOCH: Final = 1721425.5
INTERCALATION_CYCLE_YEARS: Final = 400
INTERCALATION_CYCLE_DAYS: Final = 146097
LEAP_SUPPRESSION_YEARS: Final = 100
LEAP_SUPPRESSION_DAYS: Final = 36524
LEAP_CYCLE_YEARS: Final = 4
LEAP_CYCLE_DAYS: Final = 1461
YEAR_DAYS: Final = 365
HAVE_30_DAYS: Final = (4, 6, 9, 11)
HAVE_31_DAYS: Final = (1, 3, 5, 7, 8, 10, 12)

def legal_date(year: int, month: int, day: int) -> Literal[True]: ...
def to_jd2(year: int, month: int, day: int) -> float: ...
def to_jd(year: int, month: int, day: int) -> float: ...
def from_jd(jd: float) -> tuple[int, int, int]: ...
def month_length(year: int, month: int) -> int: ...
def monthcalendar(year: int, month: int) -> list[list[int | None]]: ...
def format(year: int, month: int, day: int, format_string: str = "%-d %B %y") -> str: ...



import sys
from typing import Final

if sys.platform == "linux":
    RLIMIT_AS: Final[int]
    RLIMIT_CORE: Final[int]
    RLIMIT_CPU: Final[int]
    RLIMIT_DATA: Final[int]
    RLIMIT_FSIZE: Final[int]
    RLIMIT_LOCKS: Final[int]
    RLIMIT_MEMLOCK: Final[int]
    RLIMIT_MSGQUEUE: Final[int]
    RLIMIT_NICE: Final[int]
    RLIMIT_NOFILE: Final[int]
    RLIMIT_NPROC: Final[int]
    RLIMIT_RSS: Final[int]
    RLIMIT_RTPRIO: Final[int]
    RLIMIT_RTTIME: Final[int]
    RLIMIT_SIGPENDING: Final[int]
    RLIMIT_STACK: Final[int]
    RLIM_INFINITY: Final[int]

def getpagesize() -> int: ...
def getpriority(pid: int, /) -> int: ...
def net_if_addrs() -> list[tuple[str, int, str | None, str | None, str | None, str | None]]: ...
def net_if_flags(nic_name: str, /) -> list[str]: ...
def net_if_is_running(nic_name: str, /) -> bool: ...
def net_if_mtu(nic_name: str, /) -> int: ...

if sys.platform == "darwin":
    AF_LINK: Final[int]
    def net_if_duplex_speed(nic_name: str, /) -> list[int]: ...

def setpriority(pid: int, priority: int, /) -> None: ...



from collections.abc import Mapping
from typing import Any, TypedDict, type_check_only

@type_check_only
class _StorageData(TypedDict):
    MetadataPath: str
    TLSPath: str

@type_check_only
class _Storage(TypedDict):
    Storage: _StorageData

@type_check_only
class _Endpoint(TypedDict, total=False):
    Host: str
    SkipTLSVerify: bool

@type_check_only
class _TLSMaterial:
    TLSMaterial: dict[str, list[str]]

@type_check_only
class _MetaMetaData(TypedDict, total=False):
    StackOrchestrator: str

@type_check_only
class _Metadata(TypedDict):
    Name: str
    Metadata: _MetaMetaData
    Endpoints: Mapping[str, _Endpoint]

class Context:
    name: str
    context_type: str | None
    orchestrator: str | None
    endpoints: dict[str, _Endpoint]
    tls_cfg: dict[str, Any]
    meta_path: str
    tls_path: str
    def __init__(
        self,
        name: str,
        orchestrator: str | None = None,
        host: str | None = None,
        endpoints: Mapping[str, _Endpoint] | None = None,
        tls: bool = False,
    ) -> None: ...
    def set_endpoint(
        self,
        name: str = "docker",
        host: str | None = None,
        tls_cfg: Mapping[str, Any] | None = None,
        skip_tls_verify: bool = False,
        def_namespace: str | None = None,
    ) -> None: ...
    def inspect(self) -> Mapping[str, Any]: ...
    @classmethod
    def load_context(cls, name: str) -> Context | None: ...
    def save(self) -> None: ...
    def remove(self) -> None: ...
    def __call__(self) -> Mapping[str, Any]: ...
    def is_docker_host(self) -> bool: ...
    @property
    def Name(self) -> str: ...
    @property
    def Host(self) -> str | None: ...
    @property
    def Orchestrator(self) -> str: ...
    @property
    def Metadata(self) -> _Metadata: ...
    @property
    def TLSConfig(self) -> Any: ...
    @property
    def TLSMaterial(self) -> _TLSMaterial: ...
    @property
    def Storage(self) -> _Storage: ...



from re import Pattern
from xml.etree.ElementTree import Element

from markdown.extensions import Extension
from markdown.treeprocessors import Treeprocessor

def get_attrs_and_remainder(attrs_string: str) -> tuple[list[tuple[str, str]], str]: ...
def get_attrs(str: str) -> list[tuple[str, str]]: ...
def isheader(elem: Element) -> bool: ...

class AttrListTreeprocessor(Treeprocessor):
    BASE_RE: str
    HEADER_RE: Pattern[str]
    BLOCK_RE: Pattern[str]
    INLINE_RE: Pattern[str]
    NAME_RE: Pattern[str]
    def run(self, doc: Element) -> None: ...
    def assign_attrs(self, elem: Element, attrs_string: str, *, strict: bool = False) -> str: ...
    def sanitize_name(self, name: str) -> str: ...

class AttrListExtension(Extension): ...

def makeExtension(**kwargs) -> AttrListExtension: ...



import re
from collections.abc import Iterable, KeysView
from typing import Literal, overload

def is_ascii(s: str) -> bool: ...

class IniFile:
    defaultGroup: str
    fileExtension: str
    filename: str
    tainted: bool
    content: dict[str, dict[str, str]]
    warnings: list[str]
    errors: list[str]
    def __init__(self, filename: str | None = None) -> None: ...
    def __cmp__(self, other: IniFile) -> bool: ...
    def parse(self, filename: str, headers: Iterable[str] | None = None) -> None: ...
    @overload
    def get(
        self,
        key: str,
        group: str | None = None,
        locale: bool = False,
        type: Literal["string"] = "string",
        list: Literal[False] = False,
        strict: bool = False,
    ) -> str: ...
    @overload
    def get(
        self, key: str, group: str | None, locale: bool, type: Literal["string"], list: Literal[True], strict: bool = False
    ) -> list[str]: ...
    @overload
    def get(
        self,
        key: str,
        *,
        list: Literal[True],
        group: str | None = None,
        locale: bool = False,
        type: Literal["string"] = "string",
        strict: bool = False,
    ) -> list[str]: ...
    @overload
    def get(
        self,
        key: str,
        group: str | None,
        locale: bool,
        type: Literal["boolean"],
        list: Literal[False] = False,
        strict: bool = False,
    ) -> bool: ...
    @overload
    def get(
        self,
        key: str,
        *,
        type: Literal["boolean"],
        group: str | None = None,
        locale: bool = False,
        list: Literal[False] = False,
        strict: bool = False,
    ) -> bool: ...
    @overload
    def get(
        self, key: str, group: str | None, locale: bool, type: Literal["boolean"], list: Literal[True], strict: bool = False
    ) -> list[bool]: ...
    @overload
    def get(
        self,
        key: str,
        *,
        type: Literal["boolean"],
        list: Literal[True],
        group: str | None = None,
        locale: bool = False,
        strict: bool = False,
    ) -> list[bool]: ...
    @overload
    def get(
        self,
        key: str,
        group: str | None,
        locale: bool,
        type: Literal["integer"],
        list: Literal[False] = False,
        strict: bool = False,
    ) -> int: ...
    @overload
    def get(
        self,
        key: str,
        *,
        type: Literal["integer"],
        group: str | None = None,
        locale: bool = False,
        list: Literal[False] = False,
        strict: bool = False,
    ) -> int: ...
    @overload
    def get(
        self, key: str, group: str | None, locale: bool, type: Literal["integer"], list: Literal[True], strict: bool = False
    ) -> list[int]: ...
    @overload
    def get(
        self,
        key: str,
        *,
        type: Literal["integer"],
        list: Literal[True],
        group: str | None = None,
        locale: bool = False,
        strict: bool = False,
    ) -> list[int]: ...

    # Float
    @overload
    def get(
        self,
        key: str,
        group: str | None,
        locale: bool,
        type: Literal["numeric"],
        list: Literal[False] = False,
        strict: bool = False,
    ) -> float: ...
    @overload
    def get(
        self,
        key: str,
        *,
        type: Literal["numeric"],
        group: str | None = None,
        locale: bool = False,
        list: Literal[False] = False,
        strict: bool = False,
    ) -> float: ...
    @overload
    def get(
        self, key: str, group: str | None, locale: bool, type: Literal["numeric"], list: Literal[True], strict: bool = False
    ) -> list[float]: ...
    @overload
    def get(
        self,
        key: str,
        *,
        type: Literal["numeric"],
        list: Literal[True],
        group: str | None = None,
        locale: bool = False,
        strict: bool = False,
    ) -> list[float]: ...

    # Regex
    @overload
    def get(
        self,
        key: str,
        group: str | None,
        locale: bool,
        type: Literal["regex"],
        list: Literal[False] = False,
        strict: bool = False,
    ) -> re.Pattern[str]: ...
    @overload
    def get(
        self,
        key: str,
        *,
        type: Literal["regex"],
        group: str | None = None,
        locale: bool = False,
        list: Literal[False] = False,
        strict: bool = False,
    ) -> re.Pattern[str]: ...
    @overload
    def get(
        self, key: str, group: str | None, locale: bool, type: Literal["regex"], list: Literal[True], strict: bool = False
    ) -> list[re.Pattern[str]]: ...
    @overload
    def get(
        self,
        key: str,
        *,
        type: Literal["regex"],
        list: Literal[True],
        group: str | None = None,
        locale: bool = False,
        strict: bool = False,
    ) -> list[re.Pattern[str]]: ...
    # point
    @overload
    def get(
        self,
        key: str,
        group: str | None,
        locale: bool,
        type: Literal["point"],
        list: Literal[False] = False,
        strict: bool = False,
    ) -> tuple[int, int]: ...
    @overload
    def get(
        self,
        key: str,
        *,
        type: Literal["point"],
        group: str | None = None,
        locale: bool = False,
        list: Literal[False] = False,
        strict: bool = False,
    ) -> tuple[int, int]: ...
    @overload
    def get(
        self, key: str, group: str | None, locale: bool, type: Literal["point"], list: Literal[True], strict: bool = False
    ) -> list[tuple[int, int]]: ...
    @overload
    def get(
        self,
        key: str,
        *,
        type: Literal["point"],
        list: Literal[True],
        group: str | None = None,
        locale: bool = False,
        strict: bool = False,
    ) -> list[tuple[int, int]]: ...
    def getList(self, string: str) -> list[str]: ...
    def validate(self, report: Literal["All", "Warnings", "Errors"] = "All") -> None: ...
    def checkGroup(self, group: str) -> None: ...
    def checkKey(self, key: str, value: str, group: str) -> None: ...
    def checkValue(
        self,
        key: str,
        value: str,
        type: Literal["string", "localestring", "boolean", "numeric", "integer", "regex", "point"] = "string",
        list: bool = False,
    ) -> None: ...
    def checkExtras(self) -> None: ...
    def checkBoolean(self, value: str) -> Literal[1, 2] | None: ...
    def checkNumber(self, value: str) -> Literal[1, 2] | None: ...
    def checkInteger(self, value: str) -> Literal[1] | None: ...
    def checkPoint(self, value: str) -> Literal[1] | None: ...
    def checkString(self, value: str) -> Literal[0, 1]: ...
    def checkRegex(self, value: str) -> Literal[1] | None: ...
    def write(self, filename: str | None = None, trusted: bool = False) -> None: ...
    def set(self, key: str, value: str, group: str | None = None, locale: bool = False) -> None: ...
    def addGroup(self, group: str) -> None: ...
    def removeGroup(self, group: str) -> bool: ...
    def removeKey(self, key: str, group: str | None = None, locales: bool = True) -> str: ...
    def groups(self) -> KeysView[str]: ...
    def hasGroup(self, group: str) -> bool: ...
    def hasKey(self, key: str, group: str | None = None) -> bool: ...
    def getFileName(self) -> str: ...



from logging import Logger
from traceback import StackSummary
from typing import TypedDict, type_check_only
from typing_extensions import NotRequired

log: Logger

@type_check_only
class _StackInfo(TypedDict):
    path: str
    line: int
    label: str

@type_check_only
class _ThrowableAttrs(TypedDict):
    id: str
    message: NotRequired[str]
    type: str
    remote: bool
    stack: NotRequired[list[_StackInfo]]

class Throwable:
    id: str
    message: str
    type: str
    remote: bool
    stack: list[_StackInfo] | None
    def __init__(self, exception: Exception, stack: StackSummary, remote: bool = False) -> None: ...
    def to_dict(self) -> _ThrowableAttrs: ...



from passlib.crypto.digest import norm_hash_name as norm_hash_name

def get_prf(name): ...
def pbkdf1(secret, salt, rounds, keylen=None, hash: str = "sha1"): ...
def pbkdf2(secret, salt, rounds, keylen=None, prf: str = "hmac-sha1"): ...

__all__ = [
    # hash utils
    "norm_hash_name",
    # prf utils
    "get_prf",
    # kdfs
    "pbkdf1",
    "pbkdf2",
]



from _typeshed import Incomplete

name: str

class SourceCodeContainer:
    sourceContext: Incomplete
    text: Incomplete
    nextLineNo: int
    fileName: Incomplete
    codeContexts: Incomplete
    site: Incomplete
    startLineNumber: Incomplete
    debugDocument: Incomplete | None
    def __init__(
        self,
        text,
        fileName: str = ...,
        sourceContext: int = ...,
        startLineNumber: int = ...,
        site: Incomplete | None = ...,
        debugDocument: Incomplete | None = ...,
    ) -> None: ...
    def GetText(self): ...
    def GetName(self, dnt) -> None: ...
    def GetFileName(self): ...
    def GetPositionOfLine(self, cLineNumber): ...
    def GetLineOfPosition(self, charPos): ...
    def GetNextLine(self): ...
    def GetLine(self, num): ...
    def GetNumChars(self): ...
    def GetNumLines(self): ...
    lastPos: int
    attrs: Incomplete
    def GetSyntaxColorAttributes(self): ...
    def GetCodeContextAtPosition(self, charPos): ...

class SourceModuleContainer(SourceCodeContainer):
    module: Incomplete
    def __init__(self, module) -> None: ...
    text: Incomplete
    def GetText(self): ...
    def GetName(self, dnt): ...



from _typeshed import Incomplete

from braintree.error_result import ErrorResult
from braintree.exchange_rate_quote_payload import ExchangeRateQuotePayload
from braintree.successful_result import SuccessfulResult

class ExchangeRateQuoteGateway:
    gateway: Incomplete
    config: Incomplete
    graphql_client: Incomplete
    def __init__(self, gateway, graphql_client=None) -> None: ...
    exchange_rate_quote_payload: ExchangeRateQuotePayload
    def generate(self, request) -> SuccessfulResult | ErrorResult | None: ...



from _typeshed import Incomplete
from abc import ABCMeta
from logging import Logger

from hvac.adapters import Adapter

logger: Logger

class VaultApiBase(metaclass=ABCMeta):
    def __init__(self, adapter: Adapter[Incomplete]) -> None: ...



from _typeshed import Incomplete
from abc import abstractmethod
from collections.abc import Collection, Iterable, Sequence
from typing import Any, Literal, NamedTuple, overload
from typing_extensions import TypeAlias, Unpack

from reportlab.lib.colors import Color
from reportlab.lib.styles import PropertySet
from reportlab.lib.utils import _UNSET_
from reportlab.platypus.flowables import Flowable, _HAlignment, _VAlignment

__all__ = ("Table", "TableStyle", "CellStyle", "LongTable")

_Color: TypeAlias = Color | list[float] | tuple[float, float, float, float] | tuple[float, float, float] | str | int
# TODO: consider creating a tagged union of all the possible commands, although
#       this would restrict us to passing cmds to TableStyle.__init__ in a tuple
#       since a list would not be able to be inferred correctly
#       All commands are a tuple with a str opcode as the first element, followed
#       by the arguments for that command. Most commands start with two positions
#       indicating the cell-range affected by the command, the only exception is
#       the ROUNDEDCORNERS command which applies to the whole table always.
_SpecialRow: TypeAlias = Literal["splitfirst", "splitlast", "inrowsplitstart", "inrowsplitend"]
_TableSectionCommand: TypeAlias = tuple[str, tuple[int | _SpecialRow, int], tuple[int, int], Unpack[tuple[Any, ...]]]
_CornerRadii: TypeAlias = tuple[float, float, float, float] | list[float]
_RoundedCornersTableCommand: TypeAlias = tuple[Literal["ROUNDEDCORNERS"], _CornerRadii | None]
_TableCommand: TypeAlias = _TableSectionCommand | _RoundedCornersTableCommand

class CellStyle(PropertySet):
    name: str
    fontname: str
    fontsize: float
    leading: float
    leftPadding: float
    rightPadding: float
    topPadding: float
    bottomPadding: float
    firstLineIndent: float
    color: _Color
    alignment: Literal["LEFT", "CENTER", "CENTRE", "RIGHT", "DECIMAL"]
    background: _Color
    valign: Literal["TOP", "MIDDLE", "BOTTOM"]
    href: str | None
    direction: str | None
    shaping: Incomplete | None
    destination: Incomplete | None
    def __init__(self, name: str, parent: CellStyle | None = None) -> None: ...
    def copy(self, result: CellStyle | None = None) -> CellStyle: ...

class TableStyle:
    # TODO: Add TypedDict for Table properties that can be set through the style
    def __init__(self, cmds: Iterable[_TableCommand] | None = None, parent: TableStyle | None = None, **kw) -> None: ...
    @overload
    def add(self, *cmd: Unpack[_TableSectionCommand]) -> None: ...
    @overload
    def add(self, *cmd: Unpack[_RoundedCornersTableCommand]) -> None: ...
    def getCommands(self) -> list[_TableCommand]: ...

class ShadowStyle(NamedTuple):
    dx: int | Incomplete = 10  # TODO: is either `int` or `float`
    dy: int | Incomplete = -10  # TODO: is either `int` or `float`
    color0: _Color = "grey"
    color1: _Color = "white"
    nshades: int = 30

class Table(Flowable):
    ident: str | None
    repeatRows: int
    repeatCols: int
    splitByRow: int
    splitInRow: int
    spaceBefore: float
    spaceAfter: float
    def __init__(
        self,
        # NOTE: Technically only list or tuple works but lack of covariance
        #       on list makes this too annoying
        data: Sequence[list[Any] | tuple[Any, ...]],
        colWidths: Sequence[float | str | None] | float | str | None = None,
        rowHeights: Sequence[float | None] | float | None = None,
        style: TableStyle | Iterable[_TableCommand] | None = None,
        # docs say list/tuple, but the implementation allows any collection
        repeatRows: int | Collection[int] = 0,
        repeatCols: int | Collection[int] = 0,
        splitByRow: int = 1,
        splitInRow: int = 0,
        emptyTableAction: Literal["error", "indicate", "ignore"] | None = None,
        ident: str | None = None,
        hAlign: _HAlignment | None = None,
        vAlign: _VAlignment | None = None,
        normalizedData: int = 0,
        cellStyles: Sequence[Sequence[CellStyle]] | None = None,
        rowSplitRange: tuple[int, int] | None = None,
        spaceBefore: float | None = None,
        spaceAfter: float | None = None,
        longTableOptimize=None,
        minRowHeights: Sequence[float] | None = None,
        cornerRadii: _CornerRadii | _UNSET_ | None = ...,
        renderCB: TableRenderCB | None = None,
        shadow: ShadowStyle | None = None,
    ) -> None: ...
    def identity(self, maxLen: int | None = 30) -> str: ...
    def normalizeData(self, data: Iterable[Iterable[Any]]) -> list[list[Any]]: ...
    def minWidth(self) -> float: ...
    def setStyle(self, tblstyle: TableStyle | Iterable[_TableCommand]) -> None: ...
    def normCellRange(self, sc: int, ec: int, sr: int, er: int) -> tuple[int, int, int, int]: ...
    def onSplit(self, T: Table, byRow: int = 1) -> None: ...
    def draw(self) -> None: ...

class LongTable(Table): ...

class TableRenderCB:
    def __call__(self, T: Table, cmd: str, *args: Any) -> None: ...
    @abstractmethod
    def startTable(self, T: Table) -> None: ...
    @abstractmethod
    def startBG(self, T: Table) -> None: ...
    @abstractmethod
    def endBG(self, T: Table) -> None: ...
    @abstractmethod
    def startRow(self, T: Table, rowNo: int) -> None: ...
    @abstractmethod
    def startCell(
        self, T: Table, rowNo: int, colNo: int, cellval: Any, cellstyle: CellStyle, pos: tuple[int, int], size: tuple[int, int]
    ) -> None: ...
    @abstractmethod
    def endCell(self, T: Table) -> None: ...
    @abstractmethod
    def endRow(self, T: Table) -> None: ...
    @abstractmethod
    def startLines(self, T: Table) -> None: ...
    @abstractmethod
    def endLines(self, T: Table) -> None: ...
    @abstractmethod
    def endTable(self, T: Table) -> None: ...



import argparse
import ast
from collections.abc import Generator
from typing import Any, Final, NamedTuple
from typing_extensions import Self

class Version(NamedTuple):
    major: int = 0
    minor: int = 0
    patch: int = 0
    @classmethod
    def parse(cls, s: str) -> Self: ...

SYMBOLS: Final[list[tuple[Version, frozenset[str]]]]
VERSIONS: Final[frozenset[Version]]

class Visitor(ast.NodeVisitor):
    imports: dict[str, list[tuple[int, int]]]
    attributes: dict[str, list[tuple[int, int]]]
    defined_overload: bool
    unions_pattern_or_match: list[tuple[int, int]]
    from_imported_names: set[str]
    namedtuple_methods: list[tuple[int, int]]
    namedtuple_defaults: list[tuple[int, int]]
    def __init__(self) -> None: ...
    def visit_ImportFrom(self, node: ast.ImportFrom) -> None: ...
    def visit_Attribute(self, node: ast.Attribute) -> None: ...
    def visit_FunctionDef(self, node: ast.FunctionDef) -> None: ...
    def visit_Subscript(self, node: ast.Subscript) -> None: ...
    def visit_ClassDef(self, node: ast.ClassDef) -> None: ...
    def visit_AnnAssign(self, node: ast.AnnAssign) -> None: ...
    def generic_visit(self, node: ast.AST) -> None: ...

class Plugin:
    @staticmethod
    def add_options(option_manager: Any) -> None: ...
    @classmethod
    def parse_options(cls, options: argparse.Namespace) -> None: ...
    def __init__(self, tree: ast.AST) -> None: ...
    def run(self) -> Generator[tuple[int, int, str, type[Any]]]: ...



from typing import Any

SEARCH_CONTROLS: Any
SERVER_ENCODING: str

def random_cookie(): ...

class PagedSearchSet:
    size: Any
    response: Any
    cookie: Any
    sent: int
    done: bool
    def __init__(self, response, size, criticality) -> None: ...
    def next(self, size=None): ...

class MockBaseStrategy:
    entries: Any
    no_real_dsa: bool
    bound: Any
    custom_validators: Any
    operational_attributes: Any
    def __init__(self) -> None: ...
    def add_entry(self, dn, attributes, validate: bool = True): ...
    def remove_entry(self, dn): ...
    def entries_from_json(self, json_entry_file) -> None: ...
    def mock_bind(self, request_message, controls): ...
    def mock_delete(self, request_message, controls): ...
    def mock_add(self, request_message, controls): ...
    def mock_compare(self, request_message, controls): ...
    def mock_modify_dn(self, request_message, controls): ...
    def mock_modify(self, request_message, controls): ...
    def mock_search(self, request_message, controls): ...
    def mock_extended(self, request_message, controls): ...
    def evaluate_filter_node(self, node, candidates): ...
    def equal(self, dn, attribute_type, value_to_check): ...
    def send(self, message_type, request, controls=None): ...



import datetime
from typing import Final

EPOCH: Final = 2415020

def to_gregorian(dc: int) -> tuple[int, int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> float: ...
def to_jd(dc: int) -> float: ...
def from_jd(jdc: float) -> float: ...
def from_julian(year: int, month: int, day: int) -> float: ...
def to_julian(dc: int) -> tuple[int, int, int]: ...
def to_datetime(dc: int) -> datetime.datetime: ...
def from_datetime(date: datetime.datetime) -> float: ...



import datetime
from typing import Final

from dateutil.rrule import rrule

from ..cal import Timezone
from ..prop import vRecur
from .provider import TZProvider

__all__ = ["TZP"]

DEFAULT_TIMEZONE_PROVIDER: Final = "zoneinfo"

class TZP:
    def __init__(self, provider: str | TZProvider = "zoneinfo") -> None: ...
    def use_pytz(self) -> None: ...
    def use_zoneinfo(self) -> None: ...
    def use(self, provider: str | TZProvider) -> None: ...
    def use_default(self) -> None: ...
    def localize_utc(self, dt: datetime.date) -> datetime.datetime: ...
    def localize(self, dt: datetime.date, tz: datetime.tzinfo | str) -> datetime.datetime: ...
    def cache_timezone_component(self, timezone_component: Timezone) -> None: ...
    def fix_rrule_until(self, rrule: rrule, ical_rrule: vRecur) -> None: ...
    def create_timezone(self, timezone_component: Timezone) -> datetime.tzinfo: ...
    def clean_timezone_id(self, tzid: str) -> str: ...
    def timezone(self, tz_id: str) -> datetime.tzinfo | None: ...
    def uses_pytz(self) -> bool: ...
    def uses_zoneinfo(self) -> bool: ...
    @property
    def name(self) -> str: ...



from ..geodataframe import GeoDataFrame

def overlay(
    df1: GeoDataFrame, df2: GeoDataFrame, how: str = "intersection", keep_geom_type: bool | None = None, make_valid: bool = True
) -> GeoDataFrame: ...



from collections.abc import Collection
from typing import Literal

import numpy as np
from networkx._typing import Array2D
from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from scipy.sparse import csr_array  # type: ignore[import-untyped]  # pyright: ignore[reportMissingImports]

__all__ = [
    "laplacian_matrix",
    "normalized_laplacian_matrix",
    "directed_laplacian_matrix",
    "directed_combinatorial_laplacian_matrix",
]

@_dispatchable
def laplacian_matrix(G: Graph[_Node], nodelist: Collection[_Node] | None = None, weight: str | None = "weight") -> csr_array: ...
@_dispatchable
def normalized_laplacian_matrix(
    G: Graph[_Node], nodelist: Collection[_Node] | None = None, weight: str | None = "weight"
) -> csr_array: ...
@_dispatchable
def directed_laplacian_matrix(
    G: DiGraph[_Node],
    nodelist: Collection[_Node] | None = None,
    weight: str | None = "weight",
    walk_type: Literal["random", "lazy", "pagerank"] | None = None,
    alpha: float = 0.95,
) -> Array2D[np.float64]: ...
@_dispatchable
def directed_combinatorial_laplacian_matrix(
    G: DiGraph[_Node],
    nodelist: Collection[_Node] | None = None,
    weight: str | None = "weight",
    walk_type: Literal["random", "lazy", "pagerank"] | None = None,
    alpha: float = 0.95,
) -> Array2D[np.float64]: ...



from _typeshed import Incomplete
from collections.abc import Callable

from .base import Client, _TokenPlacement

class WebApplicationClient(Client):
    grant_type: str
    code: str | None
    def __init__(
        self,
        client_id: str,
        code: str | None = None,
        *,
        default_token_placement: _TokenPlacement = "auth_header",
        token_type: str = "Bearer",
        access_token: str | None = None,
        refresh_token: str | None = None,
        mac_key: str | bytes | bytearray | None = None,
        mac_algorithm: str | None = None,
        token: dict[str, Incomplete] | None = None,
        scope: str | set[object] | tuple[object] | list[object] | None = None,
        state: str | None = None,
        redirect_url: str | None = None,
        state_generator: Callable[[], str] = ...,
        code_verifier: str | None = None,
        code_challenge: str | None = None,
        code_challenge_method: str | None = None,
        **kwargs,
    ) -> None: ...
    def prepare_request_uri(
        self,
        uri: str,
        redirect_uri: str | None = None,
        scope: str | set[object] | tuple[object] | list[object] | None = None,
        state: str | None = None,
        code_challenge: str | None = None,
        code_challenge_method: str | None = "plain",
        **kwargs,
    ) -> str: ...
    def prepare_request_body(
        self,
        code: str | None = None,
        redirect_uri: str | None = None,
        body: str = "",
        include_client_id: bool = True,
        code_verifier: str | None = None,
        *,
        scope: str | set[object] | tuple[object] | list[object] | None = None,
        client_id: str | None = None,
        client_secret: str | None = None,
        **kwargs,
    ) -> str: ...
    def parse_request_uri_response(self, uri: str, state: str | None = None) -> dict[str, str]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
This proto file defines messages which represent the HLO module. This is a
full fidelity serialization of the c++ HLO constructs.

Many of the protos below are simple 1-to-1 serializations of the
corresponding C++ classes, e.g., HloModule, HloComputation, and
HloInstruction.

FIELD NAMES ARE IMPORTANT

Unlike most protos, you can't safely change the names of fields, even if you
keep the numeric ids the same. This is because we sometimes serialize these
protos as JSON, which includes the field names in the serialization.
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tensorflow.compiler.xla.xla_data_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _CustomCallSchedule:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _CustomCallScheduleEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_CustomCallSchedule.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SCHEDULE_NONE: _CustomCallSchedule.ValueType  # 0
    SCHEDULE_LATEST: _CustomCallSchedule.ValueType  # 1
    SCHEDULE_EARLIEST: _CustomCallSchedule.ValueType  # 2

class CustomCallSchedule(_CustomCallSchedule, metaclass=_CustomCallScheduleEnumTypeWrapper): ...

SCHEDULE_NONE: CustomCallSchedule.ValueType  # 0
SCHEDULE_LATEST: CustomCallSchedule.ValueType  # 1
SCHEDULE_EARLIEST: CustomCallSchedule.ValueType  # 2
global___CustomCallSchedule = CustomCallSchedule

class _CustomCallApiVersion:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _CustomCallApiVersionEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_CustomCallApiVersion.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    API_VERSION_UNSPECIFIED: _CustomCallApiVersion.ValueType  # 0
    API_VERSION_ORIGINAL: _CustomCallApiVersion.ValueType  # 1
    """The first version of the API, with the following signatures:

    CPU:
      void do_custom_call(void* out, const void** in);

    GPU:
      void do_custom_call(CUstream stream, void** buffers,
                          const char* opaque, size_t opaque_len);
    """
    API_VERSION_STATUS_RETURNING: _CustomCallApiVersion.ValueType  # 2
    """When the ability to return success/failure status was added:

    CPU:
      void do_custom_call(void* out, const void** in,
                          XlaCustomCallStatus* status);

    GPU:
      void do_custom_call(CUstream stream, void** buffers,
                          const char* opaque, size_t opaque_len,
                          XlaCustomCallStatus* status);
    """
    API_VERSION_STATUS_RETURNING_UNIFIED: _CustomCallApiVersion.ValueType  # 3
    """Fixes the API signatures on the CPU side of the version STATUS_RETURNING by
    adding the opaque string so that the custom call API is consistent across
    CPUs and GPUs. For GPUs, the behaviors invoked by
    API_VERSION_STATUS_RETURNING and API_VERSION_STATUS_RETURNING_UNIFIED are
    the same.

    CPU:
      void do_custom_call(void* out, const void** in,
                          const char* opaque, size_t opaque_len,
                          XlaCustomCallStatus* status);

    GPU:
      void do_custom_call(CUstream stream, void** buffers,
                          const char* opaque, size_t opaque_len,
                          XlaCustomCallStatus* status);
    """
    API_VERSION_TYPED_FFI: _CustomCallApiVersion.ValueType  # 4
    """Api version implementing XLA runtime custom call calling convention. These
    custom calls can be registered as an XLA runtime custom call (1) or as XLA
    runtime FFI binding (2).

    This type of custom call uses custom ABI to pass type information along
    with custom call arguments. Also it passes buffer arguments together with
    data type, sizes and strides.

    Example: (XLA runtime custom call)

      absl::Status DoCustomCall(StridedMemrefView arg, float attr);

      CustomCall::Bind("custom_call")
        .Arg<StridedMemrefView>()
        .Attr<float>("attr")
        .To(DoCustomCall);

    (1) xla/runtime/custom_call.h
    (2) xla/runtime/ffi/ffi.h
    """

class CustomCallApiVersion(_CustomCallApiVersion, metaclass=_CustomCallApiVersionEnumTypeWrapper):
    """The version of the API used by the custom call function. The signatures for
    each version are given below.
    TODO(b/189822916): Remove this enum when all clients are migrated to the
    status-returning API.
    """

API_VERSION_UNSPECIFIED: CustomCallApiVersion.ValueType  # 0
API_VERSION_ORIGINAL: CustomCallApiVersion.ValueType  # 1
"""The first version of the API, with the following signatures:

CPU:
  void do_custom_call(void* out, const void** in);

GPU:
  void do_custom_call(CUstream stream, void** buffers,
                      const char* opaque, size_t opaque_len);
"""
API_VERSION_STATUS_RETURNING: CustomCallApiVersion.ValueType  # 2
"""When the ability to return success/failure status was added:

CPU:
  void do_custom_call(void* out, const void** in,
                      XlaCustomCallStatus* status);

GPU:
  void do_custom_call(CUstream stream, void** buffers,
                      const char* opaque, size_t opaque_len,
                      XlaCustomCallStatus* status);
"""
API_VERSION_STATUS_RETURNING_UNIFIED: CustomCallApiVersion.ValueType  # 3
"""Fixes the API signatures on the CPU side of the version STATUS_RETURNING by
adding the opaque string so that the custom call API is consistent across
CPUs and GPUs. For GPUs, the behaviors invoked by
API_VERSION_STATUS_RETURNING and API_VERSION_STATUS_RETURNING_UNIFIED are
the same.

CPU:
  void do_custom_call(void* out, const void** in,
                      const char* opaque, size_t opaque_len,
                      XlaCustomCallStatus* status);

GPU:
  void do_custom_call(CUstream stream, void** buffers,
                      const char* opaque, size_t opaque_len,
                      XlaCustomCallStatus* status);
"""
API_VERSION_TYPED_FFI: CustomCallApiVersion.ValueType  # 4
"""Api version implementing XLA runtime custom call calling convention. These
custom calls can be registered as an XLA runtime custom call (1) or as XLA
runtime FFI binding (2).

This type of custom call uses custom ABI to pass type information along
with custom call arguments. Also it passes buffer arguments together with
data type, sizes and strides.

Example: (XLA runtime custom call)

  absl::Status DoCustomCall(StridedMemrefView arg, float attr);

  CustomCall::Bind("custom_call")
    .Arg<StridedMemrefView>()
    .Attr<float>("attr")
    .To(DoCustomCall);

(1) xla/runtime/custom_call.h
(2) xla/runtime/ffi/ffi.h
"""
global___CustomCallApiVersion = CustomCallApiVersion

class _Kind:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _KindEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Kind.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNDEFINED_ALIAS: _Kind.ValueType  # 0
    """Define a UNDEFINED_ALIAS equal to zero to get around the default-0 proto3
    behavior and missing has_*() APIs.
    """
    MAY_ALIAS: _Kind.ValueType  # 1
    """The buffers may or may not alias at runtime."""
    MUST_ALIAS: _Kind.ValueType  # 2
    """The buffers must alias at runtime."""

class Kind(_Kind, metaclass=_KindEnumTypeWrapper): ...

UNDEFINED_ALIAS: Kind.ValueType  # 0
"""Define a UNDEFINED_ALIAS equal to zero to get around the default-0 proto3
behavior and missing has_*() APIs.
"""
MAY_ALIAS: Kind.ValueType  # 1
"""The buffers may or may not alias at runtime."""
MUST_ALIAS: Kind.ValueType  # 2
"""The buffers must alias at runtime."""
global___Kind = Kind

@typing.final
class HloInstructionProto(google.protobuf.message.Message):
    """Serialization of HloInstruction.
    Next ID: 90
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class SliceDimensions(google.protobuf.message.Message):
        """Describes the [begin, end) index range and stride for slices."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        START_FIELD_NUMBER: builtins.int
        LIMIT_FIELD_NUMBER: builtins.int
        STRIDE_FIELD_NUMBER: builtins.int
        start: builtins.int
        limit: builtins.int
        stride: builtins.int
        def __init__(
            self, *, start: builtins.int | None = ..., limit: builtins.int | None = ..., stride: builtins.int | None = ...
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["limit", b"limit", "start", b"start", "stride", b"stride"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    OPCODE_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    LITERAL_FIELD_NUMBER: builtins.int
    PARAMETER_NUMBER_FIELD_NUMBER: builtins.int
    FUSION_KIND_FIELD_NUMBER: builtins.int
    TUPLE_INDEX_FIELD_NUMBER: builtins.int
    DIMENSIONS_FIELD_NUMBER: builtins.int
    WINDOW_FIELD_NUMBER: builtins.int
    CONVOLUTION_DIMENSION_NUMBERS_FIELD_NUMBER: builtins.int
    FEATURE_GROUP_COUNT_FIELD_NUMBER: builtins.int
    BATCH_GROUP_COUNT_FIELD_NUMBER: builtins.int
    SLICE_DIMENSIONS_FIELD_NUMBER: builtins.int
    EXPONENT_BITS_FIELD_NUMBER: builtins.int
    MANTISSA_BITS_FIELD_NUMBER: builtins.int
    DYNAMIC_SLICE_SIZES_FIELD_NUMBER: builtins.int
    PADDING_CONFIG_FIELD_NUMBER: builtins.int
    OUTFEED_CONFIG_FIELD_NUMBER: builtins.int
    DISTRIBUTION_FIELD_NUMBER: builtins.int
    EPSILON_FIELD_NUMBER: builtins.int
    FEATURE_INDEX_FIELD_NUMBER: builtins.int
    CHANNEL_ID_FIELD_NUMBER: builtins.int
    INFEED_CONFIG_FIELD_NUMBER: builtins.int
    CUSTOM_CALL_TARGET_FIELD_NUMBER: builtins.int
    OUTFEED_SHAPE_FIELD_NUMBER: builtins.int
    DOT_DIMENSION_NUMBERS_FIELD_NUMBER: builtins.int
    FFT_TYPE_FIELD_NUMBER: builtins.int
    FFT_LENGTH_FIELD_NUMBER: builtins.int
    COMPARISON_DIRECTION_FIELD_NUMBER: builtins.int
    GATHER_DIMENSION_NUMBERS_FIELD_NUMBER: builtins.int
    GATHER_SLICE_SIZES_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    OPERAND_IDS_FIELD_NUMBER: builtins.int
    CONTROL_PREDECESSOR_IDS_FIELD_NUMBER: builtins.int
    CALLED_COMPUTATION_IDS_FIELD_NUMBER: builtins.int
    SHARDING_FIELD_NUMBER: builtins.int
    BACKEND_CONFIG_FIELD_NUMBER: builtins.int
    REPLICA_GROUPS_FIELD_NUMBER: builtins.int
    ALL_REDUCE_ID_FIELD_NUMBER: builtins.int
    USE_GLOBAL_DEVICE_IDS_FIELD_NUMBER: builtins.int
    IS_HOST_TRANSFER_FIELD_NUMBER: builtins.int
    IS_STABLE_FIELD_NUMBER: builtins.int
    SCATTER_DIMENSION_NUMBERS_FIELD_NUMBER: builtins.int
    PRECISION_CONFIG_FIELD_NUMBER: builtins.int
    SOURCE_TARGET_PAIRS_FIELD_NUMBER: builtins.int
    DOMAIN_ENTRY_SHARDING_FIELD_NUMBER: builtins.int
    DOMAIN_EXIT_SHARDING_FIELD_NUMBER: builtins.int
    CONSTRAIN_LAYOUT_FIELD_NUMBER: builtins.int
    OPERAND_SHAPES_WITH_LAYOUT_FIELD_NUMBER: builtins.int
    TRIANGULAR_SOLVE_OPTIONS_FIELD_NUMBER: builtins.int
    CHOLESKY_OPTIONS_FIELD_NUMBER: builtins.int
    PARAMETER_REPLICATION_FIELD_NUMBER: builtins.int
    CUSTOM_CALL_HAS_SIDE_EFFECT_FIELD_NUMBER: builtins.int
    OUTPUT_OPERAND_ALIASING_FIELD_NUMBER: builtins.int
    CUSTOM_CALL_SCHEDULE_FIELD_NUMBER: builtins.int
    DELTA_FIELD_NUMBER: builtins.int
    INDICES_ARE_SORTED_FIELD_NUMBER: builtins.int
    FRONTEND_ATTRIBUTES_FIELD_NUMBER: builtins.int
    UNIQUE_INDICES_FIELD_NUMBER: builtins.int
    RNG_ALGORITHM_FIELD_NUMBER: builtins.int
    COMPARISON_TYPE_FIELD_NUMBER: builtins.int
    IS_CROSS_PROGRAM_PREFETCH_FIELD_NUMBER: builtins.int
    CROSS_PROGRAM_PREFETCH_INDEX_FIELD_NUMBER: builtins.int
    PADDING_TYPE_FIELD_NUMBER: builtins.int
    CUSTOM_CALL_API_VERSION_FIELD_NUMBER: builtins.int
    ASYNC_EXECUTION_THREAD_FIELD_NUMBER: builtins.int
    K_FIELD_NUMBER: builtins.int
    LARGEST_FIELD_NUMBER: builtins.int
    STATISTICS_VIZ_FIELD_NUMBER: builtins.int
    DOT_SPARSITY_FIELD_NUMBER: builtins.int
    COLLECTIVE_DEVICE_LIST_FIELD_NUMBER: builtins.int
    ORIGINAL_VALUE_FIELD_NUMBER: builtins.int
    IS_COMPOSITE_FIELD_NUMBER: builtins.int
    name: builtins.str
    opcode: builtins.str
    parameter_number: builtins.int
    """Parameter number is only present for kParameter."""
    fusion_kind: builtins.str
    """Fusion state, only present for kFusion."""
    tuple_index: builtins.int
    """Index for kGetTupleElement."""
    feature_group_count: builtins.int
    """The number of feature groups. Used for a convolution. Must be a divisor of
    the input feature dimension and output feature dimension. If not specified,
    it will use a default value of 1.
    """
    batch_group_count: builtins.int
    exponent_bits: builtins.int
    """The bit sizes for a reduce-precision operation."""
    mantissa_bits: builtins.int
    outfeed_config: builtins.bytes
    """Outfeed configuration information, only present for kOutfeed."""
    distribution: tensorflow.compiler.xla.xla_data_pb2.RandomDistribution.ValueType
    """The distribution requested for random number generation.
    Only present for kRng.
    """
    epsilon: builtins.float
    """A small float number added to the variance to avoid divide-by-zero error.
    Only present for kBatchNormTraining, kBatchNormInference, and
    kBatchNormGrad.
    """
    feature_index: builtins.int
    """An integer value representing the index of the feature dimension.
    Only present for kBatchNormTraining, kBatchNormInference, and
    kBatchNormGrad.
    """
    channel_id: builtins.int
    """Represents a unique identifier for each Send/Recv instruction pair or
    optionally for collective instructions (AllReduce, CollectivePermute,
    AllToAll). Non-positive channel_id is equivalent to no channel id.
    """
    infeed_config: builtins.bytes
    """The string representation of the infeed configuration."""
    custom_call_target: builtins.str
    """Name of a external target (eg, global symbol) to call, only present for
    kCustomCall.
    """
    fft_type: tensorflow.compiler.xla.xla_data_pb2.FftType.ValueType
    """FFT type (FFT, IFFT, etc)."""
    comparison_direction: builtins.str
    """Comparison direction only used for kCompare."""
    id: builtins.int
    """The id of this instruction."""
    backend_config: builtins.bytes
    """Backend configuration for the instruction. Has backend-specific meaning."""
    all_reduce_id: builtins.int
    """Deprecated, but keeping it for backward compatibility. Use channel_id.
    Non-positive all_reduce_id is equivalent to no all_reduce_id.
    """
    use_global_device_ids: builtins.bool
    """If true, interprets ids in ReplicaGroup as global device ids, which is
    a linearized id of `replica_id * partition_count + partition_id`.
    """
    is_host_transfer: builtins.bool
    """Whether this Send/Recv instruction transfers data to/from the host. Only
    present for Send and Recv instructions and their SendDone and RecvDone
    partners.
    """
    is_stable: builtins.bool
    """Whether this Sort instruction should be stable."""
    constrain_layout: builtins.bool
    """For custom call this indicates that the layouts are constrained. If
    constrain_layout is true then the 'shape' field must contain a layout, and
    'operand_shapes_with_layout' must contain a shape with layout for each
    operand.
    """
    custom_call_has_side_effect: builtins.bool
    """Whether the kCustomCall instruction has side-effects, only present for
    kCustomCall.
    """
    custom_call_schedule: global___CustomCallSchedule.ValueType
    """Specifies the desired schedule for the custom-call. The field is only
    present for custom-call.
    """
    delta: builtins.int
    """The delta value for kRngGetAndUpdateState."""
    indices_are_sorted: builtins.bool
    """Specifies if the gather/scatter indices are guaranteed to be sorted by the
    caller.
    """
    unique_indices: builtins.bool
    """Specifies if all elements updated are guaranteed to be unique by
    the caller.
    """
    rng_algorithm: tensorflow.compiler.xla.xla_data_pb2.RandomAlgorithm.ValueType
    """RNG algorithm used by kRngBitGenerator."""
    comparison_type: builtins.str
    """The comparison type used for kCompare."""
    is_cross_program_prefetch: builtins.bool
    """Specifies if this is a cross-program-prefetch, used by kCopyStart.
    Deprecated and replaced by optional_cross_program_prefetch_index.
    """
    cross_program_prefetch_index: builtins.int
    padding_type: tensorflow.compiler.xla.xla_data_pb2.PaddingType.ValueType
    """If a convolution is dynamic, a dynamic padding type will be specified."""
    custom_call_api_version: global___CustomCallApiVersion.ValueType
    """The API version used by the custom call function. This field is only
    present for custom-call.
    TODO(b/189822916): Remove this field when all clients are migrated to the
    status-returning API.
    """
    async_execution_thread: builtins.str
    """Represents a unique execution thread name for one or more async groups.
    Each HLO module may contain a main thread and one or more parallel threads.
    Empty async_execution_thread is equivalent to main thread.
    """
    k: builtins.int
    """Represents the K value for top-k."""
    largest: builtins.bool
    """Represents the largest flag for top-k."""
    is_composite: builtins.bool
    """Specifies if a call instruction is a composite."""
    @property
    def shape(self) -> tensorflow.compiler.xla.xla_data_pb2.ShapeProto: ...
    @property
    def metadata(self) -> tensorflow.compiler.xla.xla_data_pb2.OpMetadata: ...
    @property
    def literal(self) -> tensorflow.compiler.xla.xla_data_pb2.LiteralProto:
        """Literal, only present for kConstant."""

    @property
    def dimensions(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """Dimensions present for some operations that require reshaping or
        broadcasting, including Reshape, Reduce, ReduceWindow, and Reverse.
        """

    @property
    def window(self) -> tensorflow.compiler.xla.xla_data_pb2.Window:
        """Describes the window in a windowed operation such as convolution."""

    @property
    def convolution_dimension_numbers(self) -> tensorflow.compiler.xla.xla_data_pb2.ConvolutionDimensionNumbers:
        """Describes the dimension numbers used for a convolution."""

    @property
    def slice_dimensions(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___HloInstructionProto.SliceDimensions]: ...
    @property
    def dynamic_slice_sizes(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """Describes the [start, start + size) range size for a dynamic slice
        ('start' is specified dynamically in the second operand of the operation).
        """

    @property
    def padding_config(self) -> tensorflow.compiler.xla.xla_data_pb2.PaddingConfig:
        """The padding configuration that describes the edge padding and interior
        padding of this pad instruction. Only set for pad instructions.
        """

    @property
    def outfeed_shape(self) -> tensorflow.compiler.xla.xla_data_pb2.ShapeProto:
        """Shape of outfeed request."""

    @property
    def dot_dimension_numbers(self) -> tensorflow.compiler.xla.xla_data_pb2.DotDimensionNumbers:
        """Describes the dimension numbers used for a dot operation"""

    @property
    def fft_length(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """FFT length."""

    @property
    def gather_dimension_numbers(self) -> tensorflow.compiler.xla.xla_data_pb2.GatherDimensionNumbers:
        """Gather dimension numbers."""

    @property
    def gather_slice_sizes(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def operand_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def control_predecessor_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def called_computation_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def sharding(self) -> tensorflow.compiler.xla.xla_data_pb2.OpSharding: ...
    @property
    def replica_groups(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[tensorflow.compiler.xla.xla_data_pb2.ReplicaGroup]:
        """Deprecated, but keeping for backward compatibility.
        Use collective_device_list. Cross replica op fields.
        """

    @property
    def scatter_dimension_numbers(self) -> tensorflow.compiler.xla.xla_data_pb2.ScatterDimensionNumbers: ...
    @property
    def precision_config(self) -> tensorflow.compiler.xla.xla_data_pb2.PrecisionConfig:
        """Precision configuration for the instruction. Has backend-specific meaning."""

    @property
    def source_target_pairs(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[tensorflow.compiler.xla.xla_data_pb2.SourceTarget]:
        """Collective permute field."""

    @property
    def domain_entry_sharding(self) -> tensorflow.compiler.xla.xla_data_pb2.OpSharding:
        """Sharding for kDomain instructions."""

    @property
    def domain_exit_sharding(self) -> tensorflow.compiler.xla.xla_data_pb2.OpSharding: ...
    @property
    def operand_shapes_with_layout(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[tensorflow.compiler.xla.xla_data_pb2.ShapeProto]: ...
    @property
    def triangular_solve_options(self) -> tensorflow.compiler.xla.xla_data_pb2.TriangularSolveOptions:
        """Options for TriangularSolve"""

    @property
    def cholesky_options(self) -> tensorflow.compiler.xla.xla_data_pb2.CholeskyOptions:
        """Options for Cholesky"""

    @property
    def parameter_replication(self) -> tensorflow.compiler.xla.xla_data_pb2.ParameterReplication:
        """Describes how parameters behave with regards to replicas."""

    @property
    def output_operand_aliasing(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        tensorflow.compiler.xla.xla_data_pb2.OutputOperandAliasing
    ]:
        """A list of OutputOperandAliasing pairs that specifies aliasing buffers
        between output and operands for kCustomCall and kFusion.
        """

    @property
    def frontend_attributes(self) -> tensorflow.compiler.xla.xla_data_pb2.FrontendAttributes:
        """Frontend attributes to pass to the XLA backend."""

    @property
    def statistics_viz(self) -> tensorflow.compiler.xla.xla_data_pb2.StatisticsViz:
        """Represents the information for tracking propagation of values within HLO
        graph.
        """

    @property
    def dot_sparsity(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        tensorflow.compiler.xla.xla_data_pb2.SparsityDescriptor
    ]:
        """Sparsity descriptor for dot operation."""

    @property
    def collective_device_list(self) -> tensorflow.compiler.xla.xla_data_pb2.CollectiveDeviceListProto:
        """Represents the list of devices that participate in a collective operation."""

    @property
    def original_value(self) -> tensorflow.compiler.xla.xla_data_pb2.OriginalValueProto:
        """For HLO value tracking."""

    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        opcode: builtins.str | None = ...,
        shape: tensorflow.compiler.xla.xla_data_pb2.ShapeProto | None = ...,
        metadata: tensorflow.compiler.xla.xla_data_pb2.OpMetadata | None = ...,
        literal: tensorflow.compiler.xla.xla_data_pb2.LiteralProto | None = ...,
        parameter_number: builtins.int | None = ...,
        fusion_kind: builtins.str | None = ...,
        tuple_index: builtins.int | None = ...,
        dimensions: collections.abc.Iterable[builtins.int] | None = ...,
        window: tensorflow.compiler.xla.xla_data_pb2.Window | None = ...,
        convolution_dimension_numbers: tensorflow.compiler.xla.xla_data_pb2.ConvolutionDimensionNumbers | None = ...,
        feature_group_count: builtins.int | None = ...,
        batch_group_count: builtins.int | None = ...,
        slice_dimensions: collections.abc.Iterable[global___HloInstructionProto.SliceDimensions] | None = ...,
        exponent_bits: builtins.int | None = ...,
        mantissa_bits: builtins.int | None = ...,
        dynamic_slice_sizes: collections.abc.Iterable[builtins.int] | None = ...,
        padding_config: tensorflow.compiler.xla.xla_data_pb2.PaddingConfig | None = ...,
        outfeed_config: builtins.bytes | None = ...,
        distribution: tensorflow.compiler.xla.xla_data_pb2.RandomDistribution.ValueType | None = ...,
        epsilon: builtins.float | None = ...,
        feature_index: builtins.int | None = ...,
        channel_id: builtins.int | None = ...,
        infeed_config: builtins.bytes | None = ...,
        custom_call_target: builtins.str | None = ...,
        outfeed_shape: tensorflow.compiler.xla.xla_data_pb2.ShapeProto | None = ...,
        dot_dimension_numbers: tensorflow.compiler.xla.xla_data_pb2.DotDimensionNumbers | None = ...,
        fft_type: tensorflow.compiler.xla.xla_data_pb2.FftType.ValueType | None = ...,
        fft_length: collections.abc.Iterable[builtins.int] | None = ...,
        comparison_direction: builtins.str | None = ...,
        gather_dimension_numbers: tensorflow.compiler.xla.xla_data_pb2.GatherDimensionNumbers | None = ...,
        gather_slice_sizes: collections.abc.Iterable[builtins.int] | None = ...,
        id: builtins.int | None = ...,
        operand_ids: collections.abc.Iterable[builtins.int] | None = ...,
        control_predecessor_ids: collections.abc.Iterable[builtins.int] | None = ...,
        called_computation_ids: collections.abc.Iterable[builtins.int] | None = ...,
        sharding: tensorflow.compiler.xla.xla_data_pb2.OpSharding | None = ...,
        backend_config: builtins.bytes | None = ...,
        replica_groups: collections.abc.Iterable[tensorflow.compiler.xla.xla_data_pb2.ReplicaGroup] | None = ...,
        all_reduce_id: builtins.int | None = ...,
        use_global_device_ids: builtins.bool | None = ...,
        is_host_transfer: builtins.bool | None = ...,
        is_stable: builtins.bool | None = ...,
        scatter_dimension_numbers: tensorflow.compiler.xla.xla_data_pb2.ScatterDimensionNumbers | None = ...,
        precision_config: tensorflow.compiler.xla.xla_data_pb2.PrecisionConfig | None = ...,
        source_target_pairs: collections.abc.Iterable[tensorflow.compiler.xla.xla_data_pb2.SourceTarget] | None = ...,
        domain_entry_sharding: tensorflow.compiler.xla.xla_data_pb2.OpSharding | None = ...,
        domain_exit_sharding: tensorflow.compiler.xla.xla_data_pb2.OpSharding | None = ...,
        constrain_layout: builtins.bool | None = ...,
        operand_shapes_with_layout: collections.abc.Iterable[tensorflow.compiler.xla.xla_data_pb2.ShapeProto] | None = ...,
        triangular_solve_options: tensorflow.compiler.xla.xla_data_pb2.TriangularSolveOptions | None = ...,
        cholesky_options: tensorflow.compiler.xla.xla_data_pb2.CholeskyOptions | None = ...,
        parameter_replication: tensorflow.compiler.xla.xla_data_pb2.ParameterReplication | None = ...,
        custom_call_has_side_effect: builtins.bool | None = ...,
        output_operand_aliasing: (
            collections.abc.Iterable[tensorflow.compiler.xla.xla_data_pb2.OutputOperandAliasing] | None
        ) = ...,
        custom_call_schedule: global___CustomCallSchedule.ValueType | None = ...,
        delta: builtins.int | None = ...,
        indices_are_sorted: builtins.bool | None = ...,
        frontend_attributes: tensorflow.compiler.xla.xla_data_pb2.FrontendAttributes | None = ...,
        unique_indices: builtins.bool | None = ...,
        rng_algorithm: tensorflow.compiler.xla.xla_data_pb2.RandomAlgorithm.ValueType | None = ...,
        comparison_type: builtins.str | None = ...,
        is_cross_program_prefetch: builtins.bool | None = ...,
        cross_program_prefetch_index: builtins.int | None = ...,
        padding_type: tensorflow.compiler.xla.xla_data_pb2.PaddingType.ValueType | None = ...,
        custom_call_api_version: global___CustomCallApiVersion.ValueType | None = ...,
        async_execution_thread: builtins.str | None = ...,
        k: builtins.int | None = ...,
        largest: builtins.bool | None = ...,
        statistics_viz: tensorflow.compiler.xla.xla_data_pb2.StatisticsViz | None = ...,
        dot_sparsity: collections.abc.Iterable[tensorflow.compiler.xla.xla_data_pb2.SparsityDescriptor] | None = ...,
        collective_device_list: tensorflow.compiler.xla.xla_data_pb2.CollectiveDeviceListProto | None = ...,
        original_value: tensorflow.compiler.xla.xla_data_pb2.OriginalValueProto | None = ...,
        is_composite: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "cholesky_options",
            b"cholesky_options",
            "collective_device_list",
            b"collective_device_list",
            "convolution_dimension_numbers",
            b"convolution_dimension_numbers",
            "cross_program_prefetch_index",
            b"cross_program_prefetch_index",
            "domain_entry_sharding",
            b"domain_entry_sharding",
            "domain_exit_sharding",
            b"domain_exit_sharding",
            "dot_dimension_numbers",
            b"dot_dimension_numbers",
            "frontend_attributes",
            b"frontend_attributes",
            "gather_dimension_numbers",
            b"gather_dimension_numbers",
            "literal",
            b"literal",
            "metadata",
            b"metadata",
            "optional_cross_program_prefetch_index",
            b"optional_cross_program_prefetch_index",
            "original_value",
            b"original_value",
            "outfeed_shape",
            b"outfeed_shape",
            "padding_config",
            b"padding_config",
            "parameter_replication",
            b"parameter_replication",
            "precision_config",
            b"precision_config",
            "scatter_dimension_numbers",
            b"scatter_dimension_numbers",
            "shape",
            b"shape",
            "sharding",
            b"sharding",
            "statistics_viz",
            b"statistics_viz",
            "triangular_solve_options",
            b"triangular_solve_options",
            "window",
            b"window",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "all_reduce_id",
            b"all_reduce_id",
            "async_execution_thread",
            b"async_execution_thread",
            "backend_config",
            b"backend_config",
            "batch_group_count",
            b"batch_group_count",
            "called_computation_ids",
            b"called_computation_ids",
            "channel_id",
            b"channel_id",
            "cholesky_options",
            b"cholesky_options",
            "collective_device_list",
            b"collective_device_list",
            "comparison_direction",
            b"comparison_direction",
            "comparison_type",
            b"comparison_type",
            "constrain_layout",
            b"constrain_layout",
            "control_predecessor_ids",
            b"control_predecessor_ids",
            "convolution_dimension_numbers",
            b"convolution_dimension_numbers",
            "cross_program_prefetch_index",
            b"cross_program_prefetch_index",
            "custom_call_api_version",
            b"custom_call_api_version",
            "custom_call_has_side_effect",
            b"custom_call_has_side_effect",
            "custom_call_schedule",
            b"custom_call_schedule",
            "custom_call_target",
            b"custom_call_target",
            "delta",
            b"delta",
            "dimensions",
            b"dimensions",
            "distribution",
            b"distribution",
            "domain_entry_sharding",
            b"domain_entry_sharding",
            "domain_exit_sharding",
            b"domain_exit_sharding",
            "dot_dimension_numbers",
            b"dot_dimension_numbers",
            "dot_sparsity",
            b"dot_sparsity",
            "dynamic_slice_sizes",
            b"dynamic_slice_sizes",
            "epsilon",
            b"epsilon",
            "exponent_bits",
            b"exponent_bits",
            "feature_group_count",
            b"feature_group_count",
            "feature_index",
            b"feature_index",
            "fft_length",
            b"fft_length",
            "fft_type",
            b"fft_type",
            "frontend_attributes",
            b"frontend_attributes",
            "fusion_kind",
            b"fusion_kind",
            "gather_dimension_numbers",
            b"gather_dimension_numbers",
            "gather_slice_sizes",
            b"gather_slice_sizes",
            "id",
            b"id",
            "indices_are_sorted",
            b"indices_are_sorted",
            "infeed_config",
            b"infeed_config",
            "is_composite",
            b"is_composite",
            "is_cross_program_prefetch",
            b"is_cross_program_prefetch",
            "is_host_transfer",
            b"is_host_transfer",
            "is_stable",
            b"is_stable",
            "k",
            b"k",
            "largest",
            b"largest",
            "literal",
            b"literal",
            "mantissa_bits",
            b"mantissa_bits",
            "metadata",
            b"metadata",
            "name",
            b"name",
            "opcode",
            b"opcode",
            "operand_ids",
            b"operand_ids",
            "operand_shapes_with_layout",
            b"operand_shapes_with_layout",
            "optional_cross_program_prefetch_index",
            b"optional_cross_program_prefetch_index",
            "original_value",
            b"original_value",
            "outfeed_config",
            b"outfeed_config",
            "outfeed_shape",
            b"outfeed_shape",
            "output_operand_aliasing",
            b"output_operand_aliasing",
            "padding_config",
            b"padding_config",
            "padding_type",
            b"padding_type",
            "parameter_number",
            b"parameter_number",
            "parameter_replication",
            b"parameter_replication",
            "precision_config",
            b"precision_config",
            "replica_groups",
            b"replica_groups",
            "rng_algorithm",
            b"rng_algorithm",
            "scatter_dimension_numbers",
            b"scatter_dimension_numbers",
            "shape",
            b"shape",
            "sharding",
            b"sharding",
            "slice_dimensions",
            b"slice_dimensions",
            "source_target_pairs",
            b"source_target_pairs",
            "statistics_viz",
            b"statistics_viz",
            "triangular_solve_options",
            b"triangular_solve_options",
            "tuple_index",
            b"tuple_index",
            "unique_indices",
            b"unique_indices",
            "use_global_device_ids",
            b"use_global_device_ids",
            "window",
            b"window",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_cross_program_prefetch_index", b"optional_cross_program_prefetch_index"]
    ) -> typing.Literal["cross_program_prefetch_index"] | None: ...

global___HloInstructionProto = HloInstructionProto

@typing.final
class HloComputationProto(google.protobuf.message.Message):
    """Serialization of HloComputation."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    INSTRUCTIONS_FIELD_NUMBER: builtins.int
    PROGRAM_SHAPE_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    ROOT_ID_FIELD_NUMBER: builtins.int
    IS_FUSION_COMPUTATION_FIELD_NUMBER: builtins.int
    EXECUTION_THREAD_FIELD_NUMBER: builtins.int
    name: builtins.str
    id: builtins.int
    """The id of this computation."""
    root_id: builtins.int
    """The id of the root of the computation."""
    is_fusion_computation: builtins.bool
    """Whether this is a fusion computation. Fusion computations should use this
    to determine whether they are a fusion in CreateFromProto since the
    parent fusion_instruction_ may get removed and be nullptr.
    """
    execution_thread: builtins.str
    """The name of execution thread this computation belongs to."""
    @property
    def instructions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___HloInstructionProto]:
        """The array of instructions is always in a valid dependency order, where
        operands appear before their users.
        """

    @property
    def program_shape(self) -> tensorflow.compiler.xla.xla_data_pb2.ProgramShapeProto:
        """The program shape (with layout) of this computation."""

    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        instructions: collections.abc.Iterable[global___HloInstructionProto] | None = ...,
        program_shape: tensorflow.compiler.xla.xla_data_pb2.ProgramShapeProto | None = ...,
        id: builtins.int | None = ...,
        root_id: builtins.int | None = ...,
        is_fusion_computation: builtins.bool | None = ...,
        execution_thread: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["program_shape", b"program_shape"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "execution_thread",
            b"execution_thread",
            "id",
            b"id",
            "instructions",
            b"instructions",
            "is_fusion_computation",
            b"is_fusion_computation",
            "name",
            b"name",
            "program_shape",
            b"program_shape",
            "root_id",
            b"root_id",
        ],
    ) -> None: ...

global___HloComputationProto = HloComputationProto

@typing.final
class HloScheduleProto(google.protobuf.message.Message):
    """Serialization of an HLO schedule. An HLO schedule contains a total order of
    instructions for each non-fusion computation in the module.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class InstructionSequence(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        INSTRUCTION_IDS_FIELD_NUMBER: builtins.int
        @property
        def instruction_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        def __init__(self, *, instruction_ids: collections.abc.Iterable[builtins.int] | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["instruction_ids", b"instruction_ids"]) -> None: ...

    @typing.final
    class SequencesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        @property
        def value(self) -> global___HloScheduleProto.InstructionSequence: ...
        def __init__(
            self, *, key: builtins.int | None = ..., value: global___HloScheduleProto.InstructionSequence | None = ...
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    SEQUENCES_FIELD_NUMBER: builtins.int
    @property
    def sequences(
        self,
    ) -> google.protobuf.internal.containers.MessageMap[builtins.int, global___HloScheduleProto.InstructionSequence]:
        """Map from computation id to sequence."""

    def __init__(
        self, *, sequences: collections.abc.Mapping[builtins.int, global___HloScheduleProto.InstructionSequence] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["sequences", b"sequences"]) -> None: ...

global___HloScheduleProto = HloScheduleProto

@typing.final
class HloInputOutputAliasProto(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AliasEntryProto(google.protobuf.message.Message):
        """The following proto describes a pair of aliased an input
        (described by parameter number and a ShapeIndex of the parameter)
        and an output (described by a ShapeIndex of the root
        instruction). For example:

        entry = {
         output_shape_index={1},
         parameter_number=0,
         parameter_shape_index={1, 2},
        }

        This entry indicates that the first parameter's {1, 2} element is
        aliased with the {1} element of the root instruction.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        OUTPUT_SHAPE_INDEX_FIELD_NUMBER: builtins.int
        PARAMETER_NUMBER_FIELD_NUMBER: builtins.int
        PARAMETER_SHAPE_INDEX_FIELD_NUMBER: builtins.int
        KIND_FIELD_NUMBER: builtins.int
        parameter_number: builtins.int
        """Number of the parameter in entry computation."""
        kind: global___Kind.ValueType
        """The kind of alias to be setup."""
        @property
        def output_shape_index(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
            """ShapeIndex of the root hlo."""

        @property
        def parameter_shape_index(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
            """ShapeIndex of the parameter instruction."""

        def __init__(
            self,
            *,
            output_shape_index: collections.abc.Iterable[builtins.int] | None = ...,
            parameter_number: builtins.int | None = ...,
            parameter_shape_index: collections.abc.Iterable[builtins.int] | None = ...,
            kind: global___Kind.ValueType | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "kind",
                b"kind",
                "output_shape_index",
                b"output_shape_index",
                "parameter_number",
                b"parameter_number",
                "parameter_shape_index",
                b"parameter_shape_index",
            ],
        ) -> None: ...

    ENTRIES_FIELD_NUMBER: builtins.int
    @property
    def entries(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        global___HloInputOutputAliasProto.AliasEntryProto
    ]: ...
    def __init__(
        self, *, entries: collections.abc.Iterable[global___HloInputOutputAliasProto.AliasEntryProto] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["entries", b"entries"]) -> None: ...

global___HloInputOutputAliasProto = HloInputOutputAliasProto

@typing.final
class HloBufferDonorProto(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class BufferDonorEntryProto(google.protobuf.message.Message):
        """The following proto describes an input (described by parameter number and a
        ShapeIndex of the parameter) that can donate its butter to any output
        tensor. It is similar to HloInputOutputAliasProto, but without a paired
        output. For example:

        entry = {
         parameter_number=0,
         parameter_shape_index={1, 2},
        }

        This entry indicates that the first parameter's {1, 2} element can donate
        its buffer.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        PARAMETER_NUMBER_FIELD_NUMBER: builtins.int
        PARAMETER_SHAPE_INDEX_FIELD_NUMBER: builtins.int
        parameter_number: builtins.int
        """Number of the parameter in entry computation."""
        @property
        def parameter_shape_index(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
            """ShapeIndex of the parameter instruction."""

        def __init__(
            self,
            *,
            parameter_number: builtins.int | None = ...,
            parameter_shape_index: collections.abc.Iterable[builtins.int] | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "parameter_number", b"parameter_number", "parameter_shape_index", b"parameter_shape_index"
            ],
        ) -> None: ...

    ENTRIES_FIELD_NUMBER: builtins.int
    @property
    def entries(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        global___HloBufferDonorProto.BufferDonorEntryProto
    ]: ...
    def __init__(
        self, *, entries: collections.abc.Iterable[global___HloBufferDonorProto.BufferDonorEntryProto] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["entries", b"entries"]) -> None: ...

global___HloBufferDonorProto = HloBufferDonorProto

@typing.final
class CrossProgramPrefetch(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PARAMETER_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    OFFSET_FIELD_NUMBER: builtins.int
    parameter: builtins.int
    offset: builtins.int
    @property
    def index(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        parameter: builtins.int | None = ...,
        index: collections.abc.Iterable[builtins.int] | None = ...,
        offset: builtins.int | None = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["index", b"index", "offset", b"offset", "parameter", b"parameter"]
    ) -> None: ...

global___CrossProgramPrefetch = CrossProgramPrefetch

@typing.final
class StackFrameIndexProto(google.protobuf.message.Message):
    """Serialization of stack frames index representations.
    Stack frames index presented in four flat arrays:
    1. File names array.
    2. Function names array.
    3. File location array.
    4. Frame array.
    All reference ids in sub-protos are 1-based positions of the
    entity in the flat array.
    Ids are 1-based to keep 0 value as representation of non-set property.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class FileLocation(google.protobuf.message.Message):
        """Serialization of file position."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        FILE_NAME_ID_FIELD_NUMBER: builtins.int
        FUNCTION_NAME_ID_FIELD_NUMBER: builtins.int
        LINE_FIELD_NUMBER: builtins.int
        COLUMN_FIELD_NUMBER: builtins.int
        file_name_id: builtins.int
        """1-based position of file name."""
        function_name_id: builtins.int
        """1-based position of function name."""
        line: builtins.int
        """Line number."""
        column: builtins.int
        """Column number."""
        def __init__(
            self,
            *,
            file_name_id: builtins.int | None = ...,
            function_name_id: builtins.int | None = ...,
            line: builtins.int | None = ...,
            column: builtins.int | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "column", b"column", "file_name_id", b"file_name_id", "function_name_id", b"function_name_id", "line", b"line"
            ],
        ) -> None: ...

    @typing.final
    class StackFrame(google.protobuf.message.Message):
        """Serialization of frame."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        FILE_LOCATION_ID_FIELD_NUMBER: builtins.int
        PARENT_FRAME_ID_FIELD_NUMBER: builtins.int
        file_location_id: builtins.int
        """1-based position of file location."""
        parent_frame_id: builtins.int
        """1-based position of the parent frame."""
        def __init__(
            self, *, file_location_id: builtins.int | None = ..., parent_frame_id: builtins.int | None = ...
        ) -> None: ...
        def ClearField(
            self, field_name: typing.Literal["file_location_id", b"file_location_id", "parent_frame_id", b"parent_frame_id"]
        ) -> None: ...

    FILE_NAMES_FIELD_NUMBER: builtins.int
    FUNCTION_NAMES_FIELD_NUMBER: builtins.int
    FILE_LOCATIONS_FIELD_NUMBER: builtins.int
    STACK_FRAMES_FIELD_NUMBER: builtins.int
    @property
    def file_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Flat index array of file names."""

    @property
    def function_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Flat index array of function names."""

    @property
    def file_locations(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___StackFrameIndexProto.FileLocation]:
        """Flat index array of file locations."""

    @property
    def stack_frames(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___StackFrameIndexProto.StackFrame]:
        """Flat index array of frames."""

    def __init__(
        self,
        *,
        file_names: collections.abc.Iterable[builtins.str] | None = ...,
        function_names: collections.abc.Iterable[builtins.str] | None = ...,
        file_locations: collections.abc.Iterable[global___StackFrameIndexProto.FileLocation] | None = ...,
        stack_frames: collections.abc.Iterable[global___StackFrameIndexProto.StackFrame] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "file_locations",
            b"file_locations",
            "file_names",
            b"file_names",
            "function_names",
            b"function_names",
            "stack_frames",
            b"stack_frames",
        ],
    ) -> None: ...

global___StackFrameIndexProto = StackFrameIndexProto

@typing.final
class HloModuleProto(google.protobuf.message.Message):
    """Serialization of HloModule."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _ProfileType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ProfileTypeEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[HloModuleProto._ProfileType.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        INVALID: HloModuleProto._ProfileType.ValueType  # 0
        FLAG: HloModuleProto._ProfileType.ValueType  # 1
        FUSION: HloModuleProto._ProfileType.ValueType  # 2
        LAYOUT: HloModuleProto._ProfileType.ValueType  # 3
        DOT: HloModuleProto._ProfileType.ValueType  # 4
        FLAGNET: HloModuleProto._ProfileType.ValueType  # 5

    class ProfileType(_ProfileType, metaclass=_ProfileTypeEnumTypeWrapper):
        """The type of optimization profile in use for module-level optimizations."""

    INVALID: HloModuleProto.ProfileType.ValueType  # 0
    FLAG: HloModuleProto.ProfileType.ValueType  # 1
    FUSION: HloModuleProto.ProfileType.ValueType  # 2
    LAYOUT: HloModuleProto.ProfileType.ValueType  # 3
    DOT: HloModuleProto.ProfileType.ValueType  # 4
    FLAGNET: HloModuleProto.ProfileType.ValueType  # 5

    @typing.final
    class ProfileInfo(google.protobuf.message.Message):
        """Information about the optimization profile that this module contains."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        PROFILE_TYPE_FIELD_NUMBER: builtins.int
        RELATIVE_SPEEDUP_FIELD_NUMBER: builtins.int
        PROFILE_SOURCE_FIELD_NUMBER: builtins.int
        COMPILATION_EVENT_FIELD_NUMBER: builtins.int
        FINGERPRINT_FIELD_NUMBER: builtins.int
        profile_type: global___HloModuleProto.ProfileType.ValueType
        """The optimization profiles that this module contains."""
        relative_speedup: builtins.float
        """Speedup of tuned config compared to default config."""
        profile_source: tensorflow.compiler.xla.xla_data_pb2.ProfileSource.ValueType
        """The source of the optimization profile that this module contains."""
        compilation_event: tensorflow.compiler.xla.xla_data_pb2.CompilationEvent.ValueType
        """The compilation event that triggered the use of the profile."""
        fingerprint: builtins.str
        """The fingerprint of the unoptimized module this profile was applied to."""
        def __init__(
            self,
            *,
            profile_type: global___HloModuleProto.ProfileType.ValueType | None = ...,
            relative_speedup: builtins.float | None = ...,
            profile_source: tensorflow.compiler.xla.xla_data_pb2.ProfileSource.ValueType | None = ...,
            compilation_event: tensorflow.compiler.xla.xla_data_pb2.CompilationEvent.ValueType | None = ...,
            fingerprint: builtins.str | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "compilation_event",
                b"compilation_event",
                "fingerprint",
                b"fingerprint",
                "profile_source",
                b"profile_source",
                "profile_type",
                b"profile_type",
                "relative_speedup",
                b"relative_speedup",
            ],
        ) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    ENTRY_COMPUTATION_NAME_FIELD_NUMBER: builtins.int
    ENTRY_COMPUTATION_ID_FIELD_NUMBER: builtins.int
    COMPUTATIONS_FIELD_NUMBER: builtins.int
    HOST_PROGRAM_SHAPE_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    SCHEDULE_FIELD_NUMBER: builtins.int
    INPUT_OUTPUT_ALIAS_FIELD_NUMBER: builtins.int
    BUFFER_DONOR_FIELD_NUMBER: builtins.int
    CROSS_PROGRAM_PREFETCHES_FIELD_NUMBER: builtins.int
    IS_DYNAMIC_FIELD_NUMBER: builtins.int
    SPMD_OUTPUT_SHARDING_FIELD_NUMBER: builtins.int
    SPMD_PARAMETERS_SHARDINGS_FIELD_NUMBER: builtins.int
    USE_AUTO_SPMD_PARTITIONING_FIELD_NUMBER: builtins.int
    PROFILE_INFO_FIELD_NUMBER: builtins.int
    DEVICE_ASSIGNMENT_FIELD_NUMBER: builtins.int
    STACK_FRAME_INDEX_FIELD_NUMBER: builtins.int
    FRONTEND_ATTRIBUTES_FIELD_NUMBER: builtins.int
    name: builtins.str
    entry_computation_name: builtins.str
    entry_computation_id: builtins.int
    id: builtins.int
    """The id of this module."""
    is_dynamic: builtins.bool
    """True if the module contains dynamic computation."""
    use_auto_spmd_partitioning: builtins.bool
    """Uses AutoSharding pass or not."""
    @property
    def computations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___HloComputationProto]:
        """The array of computations is always in a valid dependency order, where
        callees appear before their callers.
        """

    @property
    def host_program_shape(self) -> tensorflow.compiler.xla.xla_data_pb2.ProgramShapeProto:
        """The host program shape (with layout) of the entry computation."""

    @property
    def schedule(self) -> global___HloScheduleProto:
        """The schedule for this module."""

    @property
    def input_output_alias(self) -> global___HloInputOutputAliasProto:
        """Describes alias information between inputs and outputs."""

    @property
    def buffer_donor(self) -> global___HloBufferDonorProto:
        """Describes the information of input buffer donors."""

    @property
    def cross_program_prefetches(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CrossProgramPrefetch]: ...
    @property
    def spmd_output_sharding(self) -> tensorflow.compiler.xla.xla_data_pb2.OpSharding: ...
    @property
    def spmd_parameters_shardings(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[tensorflow.compiler.xla.xla_data_pb2.OpSharding]: ...
    @property
    def profile_info(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___HloModuleProto.ProfileInfo]:
        """Profile information for the HLO module."""

    @property
    def device_assignment(self) -> tensorflow.compiler.xla.xla_data_pb2.DeviceAssignmentProto:
        """DeviceAssignment object information."""

    @property
    def stack_frame_index(self) -> global___StackFrameIndexProto:
        """Stack frames index."""

    @property
    def frontend_attributes(self) -> tensorflow.compiler.xla.xla_data_pb2.FrontendAttributes:
        """Frontend attributes to pass to the XLA backend."""

    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        entry_computation_name: builtins.str | None = ...,
        entry_computation_id: builtins.int | None = ...,
        computations: collections.abc.Iterable[global___HloComputationProto] | None = ...,
        host_program_shape: tensorflow.compiler.xla.xla_data_pb2.ProgramShapeProto | None = ...,
        id: builtins.int | None = ...,
        schedule: global___HloScheduleProto | None = ...,
        input_output_alias: global___HloInputOutputAliasProto | None = ...,
        buffer_donor: global___HloBufferDonorProto | None = ...,
        cross_program_prefetches: collections.abc.Iterable[global___CrossProgramPrefetch] | None = ...,
        is_dynamic: builtins.bool | None = ...,
        spmd_output_sharding: tensorflow.compiler.xla.xla_data_pb2.OpSharding | None = ...,
        spmd_parameters_shardings: collections.abc.Iterable[tensorflow.compiler.xla.xla_data_pb2.OpSharding] | None = ...,
        use_auto_spmd_partitioning: builtins.bool | None = ...,
        profile_info: collections.abc.Iterable[global___HloModuleProto.ProfileInfo] | None = ...,
        device_assignment: tensorflow.compiler.xla.xla_data_pb2.DeviceAssignmentProto | None = ...,
        stack_frame_index: global___StackFrameIndexProto | None = ...,
        frontend_attributes: tensorflow.compiler.xla.xla_data_pb2.FrontendAttributes | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "buffer_donor",
            b"buffer_donor",
            "device_assignment",
            b"device_assignment",
            "frontend_attributes",
            b"frontend_attributes",
            "host_program_shape",
            b"host_program_shape",
            "input_output_alias",
            b"input_output_alias",
            "schedule",
            b"schedule",
            "spmd_output_sharding",
            b"spmd_output_sharding",
            "stack_frame_index",
            b"stack_frame_index",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "buffer_donor",
            b"buffer_donor",
            "computations",
            b"computations",
            "cross_program_prefetches",
            b"cross_program_prefetches",
            "device_assignment",
            b"device_assignment",
            "entry_computation_id",
            b"entry_computation_id",
            "entry_computation_name",
            b"entry_computation_name",
            "frontend_attributes",
            b"frontend_attributes",
            "host_program_shape",
            b"host_program_shape",
            "id",
            b"id",
            "input_output_alias",
            b"input_output_alias",
            "is_dynamic",
            b"is_dynamic",
            "name",
            b"name",
            "profile_info",
            b"profile_info",
            "schedule",
            b"schedule",
            "spmd_output_sharding",
            b"spmd_output_sharding",
            "spmd_parameters_shardings",
            b"spmd_parameters_shardings",
            "stack_frame_index",
            b"stack_frame_index",
            "use_auto_spmd_partitioning",
            b"use_auto_spmd_partitioning",
        ],
    ) -> None: ...

global___HloModuleProto = HloModuleProto

@typing.final
class LogicalBufferProto(google.protobuf.message.Message):
    """Serialization of LogicalBuffer."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Location(google.protobuf.message.Message):
        """Location represents an instruction and its shape index, which uniquely
        identifies a point where a buffer is needed.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        INSTRUCTION_NAME_FIELD_NUMBER: builtins.int
        INSTRUCTION_ID_FIELD_NUMBER: builtins.int
        SHAPE_INDEX_FIELD_NUMBER: builtins.int
        instruction_name: builtins.str
        """TODO(b/239098765): Remove instruction_name and computation_name."""
        instruction_id: builtins.int
        @property
        def shape_index(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        def __init__(
            self,
            *,
            instruction_name: builtins.str | None = ...,
            instruction_id: builtins.int | None = ...,
            shape_index: collections.abc.Iterable[builtins.int] | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "instruction_id", b"instruction_id", "instruction_name", b"instruction_name", "shape_index", b"shape_index"
            ],
        ) -> None: ...

    ID_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    DEFINED_AT_FIELD_NUMBER: builtins.int
    COLOR_FIELD_NUMBER: builtins.int
    id: builtins.int
    size: builtins.int
    color: builtins.int
    @property
    def defined_at(self) -> global___LogicalBufferProto.Location:
        """The location where the buffer is defined."""

    def __init__(
        self,
        *,
        id: builtins.int | None = ...,
        size: builtins.int | None = ...,
        defined_at: global___LogicalBufferProto.Location | None = ...,
        color: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["defined_at", b"defined_at"]) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["color", b"color", "defined_at", b"defined_at", "id", b"id", "size", b"size"]
    ) -> None: ...

global___LogicalBufferProto = LogicalBufferProto

@typing.final
class BufferAllocationProto(google.protobuf.message.Message):
    """Serialization of BufferAllocation."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Assigned(google.protobuf.message.Message):
        """Assigned represents a single LogicalBuffer that is assigned to this
        BufferAllocation.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LOGICAL_BUFFER_ID_FIELD_NUMBER: builtins.int
        OFFSET_FIELD_NUMBER: builtins.int
        SIZE_FIELD_NUMBER: builtins.int
        logical_buffer_id: builtins.int
        offset: builtins.int
        size: builtins.int
        def __init__(
            self,
            *,
            logical_buffer_id: builtins.int | None = ...,
            offset: builtins.int | None = ...,
            size: builtins.int | None = ...,
        ) -> None: ...
        def ClearField(
            self, field_name: typing.Literal["logical_buffer_id", b"logical_buffer_id", "offset", b"offset", "size", b"size"]
        ) -> None: ...

    INDEX_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    IS_THREAD_LOCAL_FIELD_NUMBER: builtins.int
    IS_TUPLE_FIELD_NUMBER: builtins.int
    IS_ENTRY_COMPUTATION_PARAMETER_FIELD_NUMBER: builtins.int
    IS_CONSTANT_FIELD_NUMBER: builtins.int
    PARAMETER_NUMBER_FIELD_NUMBER: builtins.int
    PARAMETER_SHAPE_INDEX_FIELD_NUMBER: builtins.int
    MAYBE_LIVE_OUT_FIELD_NUMBER: builtins.int
    COLOR_FIELD_NUMBER: builtins.int
    ASSIGNED_FIELD_NUMBER: builtins.int
    index: builtins.int
    size: builtins.int
    is_thread_local: builtins.bool
    is_tuple: builtins.bool
    is_entry_computation_parameter: builtins.bool
    is_constant: builtins.bool
    parameter_number: builtins.int
    maybe_live_out: builtins.bool
    color: builtins.int
    @property
    def parameter_shape_index(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def assigned(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BufferAllocationProto.Assigned]: ...
    def __init__(
        self,
        *,
        index: builtins.int | None = ...,
        size: builtins.int | None = ...,
        is_thread_local: builtins.bool | None = ...,
        is_tuple: builtins.bool | None = ...,
        is_entry_computation_parameter: builtins.bool | None = ...,
        is_constant: builtins.bool | None = ...,
        parameter_number: builtins.int | None = ...,
        parameter_shape_index: collections.abc.Iterable[builtins.int] | None = ...,
        maybe_live_out: builtins.bool | None = ...,
        color: builtins.int | None = ...,
        assigned: collections.abc.Iterable[global___BufferAllocationProto.Assigned] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "assigned",
            b"assigned",
            "color",
            b"color",
            "index",
            b"index",
            "is_constant",
            b"is_constant",
            "is_entry_computation_parameter",
            b"is_entry_computation_parameter",
            "is_thread_local",
            b"is_thread_local",
            "is_tuple",
            b"is_tuple",
            "maybe_live_out",
            b"maybe_live_out",
            "parameter_number",
            b"parameter_number",
            "parameter_shape_index",
            b"parameter_shape_index",
            "size",
            b"size",
        ],
    ) -> None: ...

global___BufferAllocationProto = BufferAllocationProto

@typing.final
class HeapSimulatorTrace(google.protobuf.message.Message):
    """A trace of a HeapSimulator run."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Event(google.protobuf.message.Message):
        """The trace includes a list of events, where each event describes one action
        performed by the heap simulator.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _Kind:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _KindEnumTypeWrapper(
            google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[HeapSimulatorTrace.Event._Kind.ValueType], builtins.type
        ):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            ALLOC: HeapSimulatorTrace.Event._Kind.ValueType  # 0
            """A memory region was allocated for the buffer."""
            FREE: HeapSimulatorTrace.Event._Kind.ValueType  # 1
            """A memory region was freed for the buffer."""
            SHARE_WITH: HeapSimulatorTrace.Event._Kind.ValueType  # 2
            """A buffer was shared with another (canonical) buffer. This is similar to
            ALLOC, except that instead of allocating a new region of memory, the
            memory region of the canonical buffer is directly re-used. Multiple
            buffers may share with the same canonical buffer. The lifetime of the
            canonical buffer is extended to the union of all lifetimes.
            """

        class Kind(_Kind, metaclass=_KindEnumTypeWrapper): ...
        ALLOC: HeapSimulatorTrace.Event.Kind.ValueType  # 0
        """A memory region was allocated for the buffer."""
        FREE: HeapSimulatorTrace.Event.Kind.ValueType  # 1
        """A memory region was freed for the buffer."""
        SHARE_WITH: HeapSimulatorTrace.Event.Kind.ValueType  # 2
        """A buffer was shared with another (canonical) buffer. This is similar to
        ALLOC, except that instead of allocating a new region of memory, the
        memory region of the canonical buffer is directly re-used. Multiple
        buffers may share with the same canonical buffer. The lifetime of the
        canonical buffer is extended to the union of all lifetimes.
        """

        KIND_FIELD_NUMBER: builtins.int
        BUFFER_ID_FIELD_NUMBER: builtins.int
        COMPUTATION_NAME_FIELD_NUMBER: builtins.int
        INSTRUCTION_NAME_FIELD_NUMBER: builtins.int
        SHARE_WITH_CANONICAL_ID_FIELD_NUMBER: builtins.int
        kind: global___HeapSimulatorTrace.Event.Kind.ValueType
        buffer_id: builtins.int
        """The id of the LogicalBuffer that the event applies to."""
        computation_name: builtins.str
        """The HloInstruction that the simulation was processing that caused this
        event to occur, identified by its computation and instruction name. E.g.
        buffers defined by instruction A are allocated when processing A.
        """
        instruction_name: builtins.str
        share_with_canonical_id: builtins.int
        """The id of the canonical LogicalBuffer that the buffer shares with. Only
        set for SHARE_WITH events.
        """
        def __init__(
            self,
            *,
            kind: global___HeapSimulatorTrace.Event.Kind.ValueType | None = ...,
            buffer_id: builtins.int | None = ...,
            computation_name: builtins.str | None = ...,
            instruction_name: builtins.str | None = ...,
            share_with_canonical_id: builtins.int | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "buffer_id",
                b"buffer_id",
                "computation_name",
                b"computation_name",
                "instruction_name",
                b"instruction_name",
                "kind",
                b"kind",
                "share_with_canonical_id",
                b"share_with_canonical_id",
            ],
        ) -> None: ...

    EVENTS_FIELD_NUMBER: builtins.int
    WHOLE_MODULE_SIMULATION_FIELD_NUMBER: builtins.int
    BUFFER_ALLOCATION_INDEX_FIELD_NUMBER: builtins.int
    whole_module_simulation: builtins.bool
    buffer_allocation_index: builtins.int
    @property
    def events(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___HeapSimulatorTrace.Event]: ...
    def __init__(
        self,
        *,
        events: collections.abc.Iterable[global___HeapSimulatorTrace.Event] | None = ...,
        whole_module_simulation: builtins.bool | None = ...,
        buffer_allocation_index: builtins.int | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "buffer_allocation_index",
            b"buffer_allocation_index",
            "events",
            b"events",
            "whole_module_simulation",
            b"whole_module_simulation",
        ],
    ) -> None: ...

global___HeapSimulatorTrace = HeapSimulatorTrace

@typing.final
class HloModuleGroupProto(google.protobuf.message.Message):
    """An abstraction representing a set of HLO module built to run concurrently
    across different devices.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    HLO_MODULES_FIELD_NUMBER: builtins.int
    name: builtins.str
    @property
    def hlo_modules(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___HloModuleProto]: ...
    def __init__(
        self, *, name: builtins.str | None = ..., hlo_modules: collections.abc.Iterable[global___HloModuleProto] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["hlo_modules", b"hlo_modules", "name", b"name"]) -> None: ...

global___HloModuleGroupProto = HloModuleGroupProto

@typing.final
class BufferAssignmentProto(google.protobuf.message.Message):
    """Serialization of BufferAssignment."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class BufferAlias(google.protobuf.message.Message):
        """Alias represents a source LogicalBuffer, and the buffer location that
        aliases it.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        SOURCE_BUFFER_ID_FIELD_NUMBER: builtins.int
        LOCATION_FIELD_NUMBER: builtins.int
        source_buffer_id: builtins.int
        @property
        def location(self) -> global___LogicalBufferProto.Location: ...
        def __init__(
            self, *, source_buffer_id: builtins.int | None = ..., location: global___LogicalBufferProto.Location | None = ...
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["location", b"location"]) -> builtins.bool: ...
        def ClearField(
            self, field_name: typing.Literal["location", b"location", "source_buffer_id", b"source_buffer_id"]
        ) -> None: ...

    LOGICAL_BUFFERS_FIELD_NUMBER: builtins.int
    BUFFER_ALIASES_FIELD_NUMBER: builtins.int
    BUFFER_ALLOCATIONS_FIELD_NUMBER: builtins.int
    HEAP_SIMULATOR_TRACES_FIELD_NUMBER: builtins.int
    @property
    def logical_buffers(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___LogicalBufferProto]: ...
    @property
    def buffer_aliases(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BufferAssignmentProto.BufferAlias]: ...
    @property
    def buffer_allocations(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BufferAllocationProto]: ...
    @property
    def heap_simulator_traces(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___HeapSimulatorTrace]: ...
    def __init__(
        self,
        *,
        logical_buffers: collections.abc.Iterable[global___LogicalBufferProto] | None = ...,
        buffer_aliases: collections.abc.Iterable[global___BufferAssignmentProto.BufferAlias] | None = ...,
        buffer_allocations: collections.abc.Iterable[global___BufferAllocationProto] | None = ...,
        heap_simulator_traces: collections.abc.Iterable[global___HeapSimulatorTrace] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "buffer_aliases",
            b"buffer_aliases",
            "buffer_allocations",
            b"buffer_allocations",
            "heap_simulator_traces",
            b"heap_simulator_traces",
            "logical_buffers",
            b"logical_buffers",
        ],
    ) -> None: ...

global___BufferAssignmentProto = BufferAssignmentProto

@typing.final
class HloProto(google.protobuf.message.Message):
    """Grouping message that contains all of the information above."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HLO_MODULE_FIELD_NUMBER: builtins.int
    BUFFER_ASSIGNMENT_FIELD_NUMBER: builtins.int
    @property
    def hlo_module(self) -> global___HloModuleProto: ...
    @property
    def buffer_assignment(self) -> global___BufferAssignmentProto: ...
    def __init__(
        self, *, hlo_module: global___HloModuleProto | None = ..., buffer_assignment: global___BufferAssignmentProto | None = ...
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["buffer_assignment", b"buffer_assignment", "hlo_module", b"hlo_module"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["buffer_assignment", b"buffer_assignment", "hlo_module", b"hlo_module"]
    ) -> None: ...

global___HloProto = HloProto

@typing.final
class HloSnapshot(google.protobuf.message.Message):
    """Encapsulates HloProto together with the arguments, result, and
    execution_platform. This message is used for purposes such as
    analysis/replay/file-storage.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HLO_FIELD_NUMBER: builtins.int
    ARGUMENTS_FIELD_NUMBER: builtins.int
    RESULT_FIELD_NUMBER: builtins.int
    EXECUTION_PLATFORM_FIELD_NUMBER: builtins.int
    execution_platform: builtins.str
    """The name of the platform used to run the graph."""
    @property
    def hlo(self) -> global___HloProto:
        """The hlo graph."""

    @property
    def arguments(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[tensorflow.compiler.xla.xla_data_pb2.LiteralProto]:
        """The arguments passed to the graph."""

    @property
    def result(self) -> tensorflow.compiler.xla.xla_data_pb2.LiteralProto:
        """The result of the graph."""

    def __init__(
        self,
        *,
        hlo: global___HloProto | None = ...,
        arguments: collections.abc.Iterable[tensorflow.compiler.xla.xla_data_pb2.LiteralProto] | None = ...,
        result: tensorflow.compiler.xla.xla_data_pb2.LiteralProto | None = ...,
        execution_platform: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["hlo", b"hlo", "result", b"result"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "arguments", b"arguments", "execution_platform", b"execution_platform", "hlo", b"hlo", "result", b"result"
        ],
    ) -> None: ...

global___HloSnapshot = HloSnapshot

@typing.final
class HloModuleMetadataProto(google.protobuf.message.Message):
    """Metadata for an HLO module. Dumped after HLO passes and before LLO lowering
    with filename module_####.metadata.textproto, where #### is
    canonical_module_id.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CANONICAL_MODULE_ID_FIELD_NUMBER: builtins.int
    MODULE_GROUP_NAME_FIELD_NUMBER: builtins.int
    ORIGINAL_MODULE_ID_FIELD_NUMBER: builtins.int
    PARTITIONED_MODULE_IDS_FIELD_NUMBER: builtins.int
    PASS_METADATA_FIELD_NUMBER: builtins.int
    canonical_module_id: builtins.int
    """Uniquely identifies an HloModuleMetadata. Equal to the first unique_id
    of the module (a module may go through multiple unique_ids). If a module
    is partitioned into multiple modules, those modules will each have a new
    HloModuleMetadata with a different canonical_module_id.
    """
    module_group_name: builtins.str
    """Name of the module group that the module is part of."""
    original_module_id: builtins.int
    """The canonical module id of the module that this one is partitioned from,
    if applicable.
    """
    @property
    def partitioned_module_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """The canonical module ids of the modules that this one is partitioned into,
        if applicable.
        """

    @property
    def pass_metadata(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___HloPassMetadata]:
        """Metadata for the HLO passes that are run on the module."""

    def __init__(
        self,
        *,
        canonical_module_id: builtins.int | None = ...,
        module_group_name: builtins.str | None = ...,
        original_module_id: builtins.int | None = ...,
        partitioned_module_ids: collections.abc.Iterable[builtins.int] | None = ...,
        pass_metadata: collections.abc.Iterable[global___HloPassMetadata] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "canonical_module_id",
            b"canonical_module_id",
            "module_group_name",
            b"module_group_name",
            "original_module_id",
            b"original_module_id",
            "partitioned_module_ids",
            b"partitioned_module_ids",
            "pass_metadata",
            b"pass_metadata",
        ],
    ) -> None: ...

global___HloModuleMetadataProto = HloModuleMetadataProto

@typing.final
class HloPassMetadata(google.protobuf.message.Message):
    """Metadata for one run of an HLO pass on a module. Provides more information
    when processing debug dumps of HloProtos about the order of HLO passes and
    various other stats like duration. `pass_id` may also be used to identify a
    particular run of a pass in debug info that propagates through stages of
    compilation.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PASS_ID_FIELD_NUMBER: builtins.int
    PASS_NAME_FIELD_NUMBER: builtins.int
    PIPELINE_NAME_FIELD_NUMBER: builtins.int
    DUMP_FILENAMES_FIELD_NUMBER: builtins.int
    MODULE_CHANGED_FIELD_NUMBER: builtins.int
    MODULE_ID_FIELD_NUMBER: builtins.int
    MODULE_GROUP_MODULE_IDS_FIELD_NUMBER: builtins.int
    START_TIMESTAMP_USEC_FIELD_NUMBER: builtins.int
    END_TIMESTAMP_USEC_FIELD_NUMBER: builtins.int
    CUSTOM_METADATA_FIELD_NUMBER: builtins.int
    pass_id: builtins.int
    """For a given module, pass_id uniquely identifies a run of an HLO pass on
    that module. Note that a pass_id may not always refer to the same pass
    because the order of passes during compilation may change. For finding
    metadata for a particular pass, pass_name and pipeline_name would be more
    reliable, although note that they may not be unique.
    """
    pass_name: builtins.str
    pipeline_name: builtins.str
    module_changed: builtins.bool
    """Return value of pass.Run(). True if this pass changed the module, or, in
    the case where the module was run through this pass as part of a module
    group, true if this pass changed any module in the same module group.
    """
    module_id: builtins.int
    """The unique_id of the module that this pass is run on. May be different from
    the canonical_module_id of the HloModuleMetadata that this HloPassMetadata
    is inside.
    """
    start_timestamp_usec: builtins.int
    """Timestamp before and after the pass is run. Note they may be equal."""
    end_timestamp_usec: builtins.int
    @property
    def dump_filenames(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Filenames of the dumps of the module after this pass ran. Module may be
        dumped in multiple formats, and the order of formats in this field will
        stay consistent across passes.
        """

    @property
    def module_group_module_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """If the module went through this pass as part of a module group, this is
        set as the ids of all the modules in the module group. Empty otherwise.
        """

    @property
    def custom_metadata(self) -> google.protobuf.any_pb2.Any:
        """Custom metadata for the pass."""

    def __init__(
        self,
        *,
        pass_id: builtins.int | None = ...,
        pass_name: builtins.str | None = ...,
        pipeline_name: builtins.str | None = ...,
        dump_filenames: collections.abc.Iterable[builtins.str] | None = ...,
        module_changed: builtins.bool | None = ...,
        module_id: builtins.int | None = ...,
        module_group_module_ids: collections.abc.Iterable[builtins.int] | None = ...,
        start_timestamp_usec: builtins.int | None = ...,
        end_timestamp_usec: builtins.int | None = ...,
        custom_metadata: google.protobuf.any_pb2.Any | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["custom_metadata", b"custom_metadata"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "custom_metadata",
            b"custom_metadata",
            "dump_filenames",
            b"dump_filenames",
            "end_timestamp_usec",
            b"end_timestamp_usec",
            "module_changed",
            b"module_changed",
            "module_group_module_ids",
            b"module_group_module_ids",
            "module_id",
            b"module_id",
            "pass_id",
            b"pass_id",
            "pass_name",
            b"pass_name",
            "pipeline_name",
            b"pipeline_name",
            "start_timestamp_usec",
            b"start_timestamp_usec",
        ],
    ) -> None: ...

global___HloPassMetadata = HloPassMetadata



from logging import Logger

from pika.adapters.base_connection import BaseConnection
from pika.adapters.utils.nbio_interface import AbstractIOReference
from pika.adapters.utils.selector_ioloop_adapter import AbstractSelectorIOLoop, SelectorIOServicesAdapter

LOGGER: Logger

class GeventConnection(BaseConnection):
    def __init__(
        self,
        parameters=None,
        on_open_callback=None,
        on_open_error_callback=None,
        on_close_callback=None,
        custom_ioloop=None,
        internal_connection_workflow: bool = True,
    ) -> None: ...
    @classmethod
    def create_connection(cls, connection_configs, on_done, custom_ioloop=None, workflow=None): ...

class _TSafeCallbackQueue:
    def __init__(self) -> None: ...
    @property
    def fd(self): ...
    def add_callback_threadsafe(self, callback) -> None: ...
    def run_next_callback(self) -> None: ...

class _GeventSelectorIOLoop(AbstractSelectorIOLoop):
    READ: int
    WRITE: int
    ERROR: int
    def __init__(self, gevent_hub=None) -> None: ...
    def close(self) -> None: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...
    def add_callback(self, callback) -> None: ...
    def call_later(self, delay, callback): ...
    def remove_timeout(self, timeout_handle) -> None: ...
    def add_handler(self, fd, handler, events) -> None: ...
    def update_handler(self, fd, events) -> None: ...
    def remove_handler(self, fd) -> None: ...

class _GeventSelectorIOServicesAdapter(SelectorIOServicesAdapter):
    def getaddrinfo(self, host, port, on_done, family: int = 0, socktype: int = 0, proto: int = 0, flags: int = 0): ...

class _GeventIOLoopIOHandle(AbstractIOReference):
    def __init__(self, subject) -> None: ...
    def cancel(self): ...

class _GeventAddressResolver:
    __slots__ = ("_loop", "_on_done", "_greenlet", "_ga_host", "_ga_port", "_ga_family", "_ga_socktype", "_ga_proto", "_ga_flags")
    def __init__(self, native_loop, host, port, family, socktype, proto, flags, on_done) -> None: ...
    def start(self) -> None: ...
    def cancel(self): ...



from gevent.hub import Hub
from greenlet import GreenletExit

class LoopExit(Exception):
    @property
    def hub(self) -> Hub | None: ...

class BlockingSwitchOutError(AssertionError): ...
class InvalidSwitchError(AssertionError): ...
class ConcurrentObjectUseError(AssertionError): ...
class InvalidThreadUseError(RuntimeError): ...

class HubDestroyed(GreenletExit):
    destroy_loop: bool
    def __init__(self, destroy_loop: bool) -> None: ...

__all__ = ["LoopExit"]



__all__ = ["Sequencer", "getSequencer", "setSequencer"]

class _Counter:
    def __init__(self) -> None: ...
    def setFormatter(self, formatFunc) -> None: ...
    def reset(self, value=None) -> None: ...
    def next(self): ...
    __next__ = next
    def nextf(self): ...
    def thisf(self): ...
    def chain(self, otherCounter) -> None: ...

class Sequencer:
    def __init__(self) -> None: ...
    def __next__(self): ...
    def next(self, counter=None): ...
    def thisf(self, counter=None): ...
    def nextf(self, counter=None): ...
    def setDefaultCounter(self, default=None) -> None: ...
    def registerFormat(self, format, func) -> None: ...
    def setFormat(self, counter, format) -> None: ...
    def reset(self, counter=None, base: int = 0) -> None: ...
    def chain(self, parent, child) -> None: ...
    def __getitem__(self, key): ...
    def format(self, template): ...
    def dump(self) -> None: ...

def getSequencer(): ...
def setSequencer(seq): ...



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["dedensify", "snap_aggregation"]

@_dispatchable
def dedensify(G: Graph[_Node], threshold: int, prefix=None, copy: bool | None = True): ...
@_dispatchable
def snap_aggregation(
    G: Graph[_Node],
    node_attributes,
    edge_attributes: Iterable[Incomplete] | None = (),
    prefix: str = "Supernode-",
    supernode_attribute: str = "group",
    superedge_attribute: str = "types",
): ...



from typing import Final, Literal

EPOCH: Final = 1948439.5
WEEKDAYS: Final = ("al-'ahad", "al-'ithnayn", "ath-thalatha'", "al-'arb`a'", "al-khamis", "al-jum`a", "as-sabt")
MONTHS: Final[list[str]]
HAS_29_DAYS: Final = (2, 4, 6, 8, 10)
HAS_30_DAYS: Final = (1, 3, 5, 7, 9, 11)

def leap(year: int) -> bool: ...
def to_jd(year: int, month: int, day: int) -> float: ...
def from_jd(jd: float) -> tuple[int, int, int]: ...
def to_jd_gregorianyear(gregorianyear: int, islamic_month: int, islamic_day: int) -> float: ...
def from_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def to_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def month_length(year: int, month: int) -> Literal[29, 30]: ...
def monthcalendar(year: int, month: int) -> list[list[int | None]]: ...
def format(year: int, month: int, day: int) -> str: ...



from _typeshed import Incomplete
from typing import Any, ClassVar

class BaseClaims(dict[str, Any]):  # dict values are key-dependent
    REGISTERED_CLAIMS: ClassVar[list[str]]
    header: Incomplete
    options: Incomplete
    params: Incomplete
    def __init__(self, payload, header, options=None, params=None) -> None: ...
    # TODO: Adds an attribute for each key in REGISTERED_CLAIMS
    def __getattr__(self, key: str): ...
    def get_registered_claims(self) -> dict[str, Incomplete]: ...

class JWTClaims(BaseClaims):
    def validate(self, now=None, leeway: int = 0) -> None: ...
    def validate_iss(self) -> None: ...
    def validate_sub(self) -> None: ...
    def validate_aud(self) -> None: ...
    def validate_exp(self, now, leeway) -> None: ...
    def validate_nbf(self, now, leeway) -> None: ...
    def validate_iat(self, now, leeway) -> None: ...
    def validate_jti(self) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar

class SignatureVerifier:
    DISABLE_JWT_CHECKS: ClassVar[dict[str, bool]]
    def __init__(self, algorithm: str) -> None: ...
    async def verify_signature(self, token: str) -> dict[str, Incomplete]: ...

class SymmetricSignatureVerifier(SignatureVerifier):
    def __init__(self, shared_secret: str, algorithm: str = "HS256") -> None: ...

class JwksFetcher:
    CACHE_TTL: ClassVar[int]
    def __init__(self, jwks_url: str, cache_ttl: int = 600) -> None: ...
    def get_key(self, key_id: str): ...

class AsymmetricSignatureVerifier(SignatureVerifier):
    def __init__(self, jwks_url: str, algorithm: str = "RS256", cache_ttl: int = 600) -> None: ...

class TokenVerifier:
    iss: str
    aud: str
    leeway: int
    def __init__(self, signature_verifier: SignatureVerifier, issuer: str, audience: str, leeway: int = 0) -> None: ...
    def verify(
        self, token: str, nonce: str | None = None, max_age: int | None = None, organization: str | None = None
    ) -> dict[str, Incomplete]: ...



from collections.abc import Sequence
from typing import Final

__version__: Final[str]
STARTUP: Final[Sequence[str]]
COMPUTERS: Final[Sequence[str]]
BLAH: Final[Sequence[str]]
BUZZWORD: Final[Sequence[str]]
STARTREK: Final[Sequence[str]]
PRINTING: Final[Sequence[str]]
PYTHON: Final[Sequence[str]]
leadins: Final[Sequence[str]]
subjects: Final[Sequence[str]]
verbs: Final[Sequence[str]]
objects: Final[Sequence[str]]

def format_wisdom(text: str, line_length: int = 72) -> str: ...
def chomsky(times: int = 1) -> str: ...
def randomText(theme: str | Sequence[str] = ..., sentences: int = 5) -> str: ...



from collections.abc import Callable, Iterable, Iterator, Sequence
from typing import Any, Generic, TypeVar

from wtforms.fields.core import Field, UnboundField, _FormT, _Validator, _Widget
from wtforms.form import BaseForm
from wtforms.meta import DefaultMeta, _SupportsGettextAndNgettext

__all__ = ("FieldList",)

_BoundFieldT = TypeVar("_BoundFieldT", bound=Field)

class FieldList(Field, Generic[_BoundFieldT]):
    unbound_field: UnboundField[_BoundFieldT]
    min_entries: int
    max_entries: int | None
    last_index: int
    entries: list[_BoundFieldT]
    object_data: Iterable[Any]
    # NOTE: This depends on the shape of errors of the bound field, which usually should
    #       be a `Sequence[Sequence[str]]`, but can be `Sequence[_FormErrors]` for `FormField`
    #       we could model this with a fake descriptor with overloads for `FieldList[FormField]`
    #       but it might not be worth the hassle, for now we'll just leave it lax
    errors: Sequence[Any]
    def __init__(
        self: FieldList[_BoundFieldT],  # pyright: ignore[reportInvalidTypeVarUse]  #11780
        # because of our workaround we need to accept Field as well
        unbound_field: UnboundField[_BoundFieldT] | _BoundFieldT,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, _BoundFieldT], ...] | list[Any] | None = None,
        min_entries: int = 0,
        max_entries: int | None = None,
        separator: str = "-",
        default: Iterable[Any] | Callable[[], Iterable[Any]] = (),
        *,
        description: str = "",
        id: str | None = None,
        widget: _Widget[FieldList[Any]] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...
    def append_entry(self, data: Any = ...) -> _BoundFieldT: ...
    def pop_entry(self) -> _BoundFieldT: ...
    def __iter__(self) -> Iterator[_BoundFieldT]: ...
    def __len__(self) -> int: ...
    def __getitem__(self, index: int) -> _BoundFieldT: ...
    @property
    def data(self) -> list[Any]: ...



from .ansi import Back as Back, Cursor as Cursor, Fore as Fore, Style as Style
from .ansitowin32 import AnsiToWin32 as AnsiToWin32
from .initialise import (
    colorama_text as colorama_text,
    deinit as deinit,
    init as init,
    just_fix_windows_console as just_fix_windows_console,
    reinit as reinit,
)



import datetime
from _typeshed import Incomplete
from collections.abc import Callable
from typing import Literal

from oauthlib.common import Request, _HTTPMethod
from oauthlib.oauth2.rfc6749.request_validator import RequestValidator

class OAuth2Token(dict[str, Incomplete]):
    def __init__(
        self, params: dict[str, Incomplete], old_scope: str | set[object] | tuple[object] | list[object] | None = None
    ) -> None: ...
    @property
    def scope_changed(self) -> bool: ...
    @property
    def old_scope(self) -> str | None: ...
    @property
    def old_scopes(self) -> list[str]: ...
    @property
    def scope(self) -> str | None: ...
    @property
    def scopes(self) -> list[str]: ...
    @property
    def missing_scopes(self) -> list[str]: ...
    @property
    def additional_scopes(self) -> list[str]: ...

def prepare_mac_header(
    token: str,
    uri: str,
    key: str | bytes | bytearray,
    http_method: _HTTPMethod,
    nonce: str | None = None,
    headers: dict[str, str] | None = None,
    body: str | None = None,
    ext: str = "",
    hash_algorithm: str = "hmac-sha-1",
    issue_time: datetime.datetime | None = None,
    draft: int = 0,
) -> dict[str, str]: ...
def prepare_bearer_uri(token: str, uri: str) -> str: ...
def prepare_bearer_headers(token: str, headers: dict[str, str] | None = None) -> dict[str, str]: ...
def prepare_bearer_body(token: str, body: str = "") -> str: ...
def random_token_generator(request: Request, refresh_token: bool = False) -> str: ...
def signed_token_generator(private_pem: str, **kwargs) -> Callable[[Request], str]: ...
def get_token_from_header(request: Request) -> str | None: ...

class TokenBase:
    __slots__ = ()
    def __call__(self, request: Request, refresh_token: bool = False) -> None: ...
    def validate_request(self, request: Request) -> bool: ...
    def estimate_type(self, request: Request) -> int: ...

class BearerToken(TokenBase):
    __slots__ = ("request_validator", "token_generator", "refresh_token_generator", "expires_in")
    request_validator: RequestValidator | None
    token_generator: Callable[[Request], str]
    refresh_token_generator: Callable[[Request], str]
    expires_in: int | Callable[[Request], int]
    def __init__(
        self,
        request_validator: RequestValidator | None = None,
        token_generator: Callable[[Request], str] | None = None,
        expires_in: int | Callable[[Request], int] | None = None,
        refresh_token_generator: Callable[[Request], str] | None = None,
    ) -> None: ...
    def create_token(self, request: Request, refresh_token: bool = False, **kwargs) -> OAuth2Token: ...
    def validate_request(self, request: Request) -> bool: ...
    def estimate_type(self, request: Request) -> Literal[9, 5, 0]: ...
